@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -27,31 +28,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -69,15 +71,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -88,7 +91,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_misc.c 13660 2009-06-10 18:45:47Z david $ */
+/* $Id: nbase_misc.c 28192 2012-03-01 06:53:35Z fyodor $ */
 
 #include "nbase.h"
 
@@ -101,6 +104,8 @@ extern int errno;
 #include <winsock2.h>
 #endif
 
+#include <limits.h>
+#include <stdio.h>
 #include "nbase_ipv6.h"
 #include "nbase_crc32ct.h"
 
@@ -137,7 +142,9 @@ char *socket_strerror(int errnum) {
 #ifdef WIN32
 	static char buffer[128];
 
-	FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
+	FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM |
+		FORMAT_MESSAGE_IGNORE_INSERTS |
+		FORMAT_MESSAGE_MAX_WIDTH_MASK,
 		0, errnum, 0, buffer, sizeof(buffer), NULL);
 
 	return buffer;
@@ -146,17 +153,52 @@ char *socket_strerror(int errnum) {
 #endif
 }
 
+/* Compares two sockaddr_storage structures with a return value like strcmp.
+   First the address families are compared, then the addresses if the families
+   are equal. The structures must be real full-length sockaddr_storage
+   structures, not something shorter like sockaddr_in. */
+int sockaddr_storage_cmp(const struct sockaddr_storage *a,
+  const struct sockaddr_storage *b) {
+  if (a->ss_family < b->ss_family)
+    return -1;
+  else if (a->ss_family > b->ss_family)
+    return 1;
+  if (a->ss_family == AF_INET) {
+    struct sockaddr_in *sin_a = (struct sockaddr_in *) a;
+    struct sockaddr_in *sin_b = (struct sockaddr_in *) b;
+    if (sin_a->sin_addr.s_addr < sin_b->sin_addr.s_addr)
+      return -1;
+    else if (sin_a->sin_addr.s_addr > sin_b->sin_addr.s_addr)
+      return 1;
+    else
+      return 0;
+  } else if (a->ss_family == AF_INET6) {
+    struct sockaddr_in6 *sin6_a = (struct sockaddr_in6 *) a;
+    struct sockaddr_in6 *sin6_b = (struct sockaddr_in6 *) b;
+    return memcmp(sin6_a->sin6_addr.s6_addr, sin6_b->sin6_addr.s6_addr,
+                  sizeof(sin6_a->sin6_addr.s6_addr));
+  } else {
+    assert(0);
+  }
+  return 0; /* Not reached */
+}
+
+int sockaddr_storage_equal(const struct sockaddr_storage *a,
+  const struct sockaddr_storage *b) {
+  return sockaddr_storage_cmp(a, b) == 0;
+}
+
 /* This function is an easier version of inet_ntop because you don't
    need to pass a dest buffer.  Instead, it returns a static buffer that
    you can use until the function is called again (by the same or another
    thread in the process).  If there is a wierd error (like sslen being
    too short) then NULL will be returned. */
-const char *inet_ntop_ez(struct sockaddr_storage *ss, size_t sslen) {
+const char *inet_ntop_ez(const struct sockaddr_storage *ss, size_t sslen) {
 
-  struct sockaddr_in *sin = (struct sockaddr_in *) ss;
+  const struct sockaddr_in *sin = (struct sockaddr_in *) ss;
   static char str[INET6_ADDRSTRLEN];
 #if HAVE_IPV6
-  struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) ss;
+  const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) ss;
 #endif
 
   str[0] = '\0';
@@ -191,6 +233,24 @@ int inheritable_socket(int af, int style, int protocol) {
 #endif
 }
 
+/* The dup function on Windows works only on file descriptors, not socket
+   handles. This function accomplishes the same thing for sockets. */
+int dup_socket(int sd) {
+#ifdef WIN32
+  HANDLE copy;
+
+  if (DuplicateHandle(GetCurrentProcess(), (HANDLE) sd,
+                      GetCurrentProcess(), &copy,
+                      0, FALSE, DUPLICATE_SAME_ACCESS) == 0) {
+    return -1;
+  }
+
+  return (int) copy;
+#else
+  return dup(sd);
+#endif
+}
+
 int unblock_socket(int sd) {
 #ifdef WIN32
 u_long one = 1;
@@ -221,27 +281,59 @@ int block_socket(int sd) {
   return 1;
 }
 
-/* Converts a time specification string into milliseconds.  If the string
- * is a plain non-negative number, it is considered to already be in
- * milliseconds and is returned.  If it is a number followed by 's' (for
- * seconds), 'm' (minutes), or 'h' (hours), the number is converted to
- * milliseconds and returned.  If it cannot parse the string, -1 is
- * returned instead.
- */
-long tval2msecs(char *tspec) {
-  long l;
-  char *endptr = NULL;
-  l = strtol(tspec, &endptr, 10);
-  if (l < 0 || !endptr) return -1;
-  if (*endptr == '\0') return l;
-  if (*endptr == 's' || *endptr == 'S') return l * 1000;
-  if ((*endptr == 'm' || *endptr == 'M')) {
-    if (*(endptr + 1) == 's' || *(endptr + 1) == 'S') 
-      return l;
-    return l * 60000;
-  }
-  if (*endptr == 'h' || *endptr == 'H') return l * 3600000;
-  return -1;
+/* Convert a time specification into a count of seconds. A time specification is
+ * a non-negative real number, possibly followed by a units suffix. The suffixes
+ * are "ms" for milliseconds, "s" for seconds, "m" for minutes, or "h" for
+ * hours. Seconds is the default with no suffix. -1 is returned if the string
+ * can't be parsed. */
+double tval2secs(const char *tspec) {
+  double d;
+  char *tail;
+
+  errno = 0;
+  d = strtod(tspec, &tail);
+  if (*tspec == '\0' || errno != 0)
+    return -1;
+  if (strcasecmp(tail, "ms") == 0)
+    return d / 1000.0;
+  else if (*tail == '\0' || strcasecmp(tail, "s") == 0)
+    return d;
+  else if (strcasecmp(tail, "m") == 0)
+    return d * 60.0;
+  else if (strcasecmp(tail, "h") == 0)
+    return d * 60.0 * 60.0;
+  else
+    return -1;
+}
+
+long tval2msecs(const char *tspec) {
+  double s, ms;
+
+  s = tval2secs(tspec);
+  if (s == -1)
+    return -1;
+  ms = s * 1000.0;
+  if (ms > LONG_MAX || ms < LONG_MIN)
+    return -1;
+
+  return (long) ms;
+}
+
+/* Returns the unit portion of a time specification (such as "ms", "s", "m", or
+   "h"). Returns NULL if there was a parsing error or no unit is present. */
+const char *tval_unit(const char *tspec) {
+  double d;
+  char *tail;
+
+  errno = 0;
+  d = strtod(tspec, &tail);
+  /* Avoid GCC 4.6 error "variable 'd' set but not used
+     [-Wunused-but-set-variable]". */
+  (void) d;
+  if (*tspec == '\0' || errno != 0 || *tail == '\0')
+    return NULL;
+
+  return tail;
 }
 
 /* A replacement for select on Windows that allows selecting on stdin
@@ -302,7 +394,8 @@ int fselect(int s, fd_set *rmaster, fd_set *wmaster, fd_set *emaster, struct tim
      * stdin has input. It involves a background thread, which we start
      * now if necessary. */
     if (!stdin_thread_started) {
-        assert(win_stdin_start_thread() != 0);
+        int ret = win_stdin_start_thread();
+        assert(ret != 0);
         stdin_thread_started = 1;
     }
 
@@ -362,8 +455,18 @@ int fselect(int s, fd_set *rmaster, fd_set *wmaster, fd_set *emaster, struct tim
 
 /*
  * CRC32 Cyclic Redundancy Check
- * simply copied from http://www.ptb.de/de/org/1/11/112/infos/crc16.htm
- * (I don't know why he calls it crc16 there, as it returns a 32-bit result)
+ *
+ * From: http://www.ietf.org/rfc/rfc1952.txt
+ *
+ * Copyright (c) 1996 L. Peter Deutsch
+ *
+ * Permission is granted to copy and distribute this document for any
+ * purpose and without charge, including translations into other
+ * languages and incorporation into compilations, provided that the
+ * copyright notice and this notice are preserved, and that any
+ * substantive changes or deletions from the original are clearly
+ * marked.
+ *
  */
 
 /* Table of CRCs of all 8-bit messages. */
@@ -502,3 +605,233 @@ unsigned long nbase_adler32(unsigned char *buf, int len)
 
 #undef ADLER32_BASE
 
+
+/* This function returns a string containing the hexdump of the supplied
+ * buffer. It uses current locale to determine if a character is printable or
+ * not. It prints 73char+\n wide lines like these:
+
+0000   e8 60 65 86 d7 86 6d 30  35 97 54 87 ff 67 05 9e  .`e...m05.T..g.. 
+0010   07 5a 98 c0 ea ad 50 d2  62 4f 7b ff e1 34 f8 fc  .Z....P.bO{..4.. 
+0020   c4 84 0a 6a 39 ad 3c 10  63 b2 22 c4 24 40 f4 b1  ...j9.<.c.".$@.. 
+
+ * The lines look basically like Wireshark's hex dump.
+ * WARNING: This function returns a pointer to a DYNAMICALLY allocated buffer
+ * that the caller is supposed to free().
+ * */
+char *hexdump(const u8 *cp, u32 length){
+  static char asciify[257];          /* Stores cha6acter table           */
+  int asc_init=0;                    /* Flag to generate table only once */
+  u32 i=0, hex=0, asc=0;             /* Array indexes                    */
+  u32 line_count=0;                  /* For byte count at line start     */
+  char *current_line=NULL;           /* Current line to write            */
+  char *buffer=NULL;                 /* Dynamic buffer we return         */
+  #define LINE_LEN 74                /* Lenght of printed line           */
+  char line2print[LINE_LEN];         /* Stores current line              */
+  char printbyte[16];                /* For byte conversion              */
+  int bytes2alloc;                   /* For buffer                       */
+  memset(line2print, ' ', LINE_LEN); /* We fill the line with spaces     */
+
+  /* On the first run, generate a list of nice printable characters
+   * (according to current locale) */
+  if( asc_init==0){
+      asc_init=1;
+      for(i=0; i<256; i++){
+        if( isalnum(i) || isdigit(i) || ispunct(i) ){ asciify[i]=i; }
+        else{ asciify[i]='.'; }
+      }
+  }
+  /* Allocate enough space to print the hex dump */
+  bytes2alloc=(length%16==0)? (1 + LINE_LEN * (length/16)) : (1 + LINE_LEN * (1+(length/16))) ;
+  buffer=(char *)safe_zalloc(bytes2alloc);
+  current_line=buffer;
+#define HEX_START 7
+#define ASC_START 57
+/* This is how or line looks like.
+0000   00 01 02 03 04 05 06 07  08 09 0a 0b 0c 0d 0e 0f  .`e...m05.T..g..[\n]
+01234567890123456789012345678901234567890123456789012345678901234567890123
+0         1         2         3         4         5         6         7
+       ^                                                 ^               ^
+       |                                                 |               |
+    HEX_START                                        ASC_START        Newline
+*/
+  i=0;
+  while( i < length ){
+    memset(line2print, ' ', LINE_LEN); /* Fill line with spaces */
+    snprintf(line2print, sizeof(line2print), "%04x", (16*line_count++) % 0xFFFF); /* Add line No.*/
+    line2print[4]=' '; /* Replace the '\0' inserted by snprintf() with a space */
+    hex=HEX_START;  asc=ASC_START;
+    do { /* Print 16 bytes in both hex and ascii */
+		if (i%16 == 8) hex++; /* Insert space every 8 bytes */
+        snprintf(printbyte, sizeof(printbyte), "%02x", cp[i]);/* First print the hex number */
+        line2print[hex++]=printbyte[0];
+        line2print[hex++]=printbyte[1];
+        line2print[hex++]=' ';
+        line2print[asc++]=asciify[ cp[i] ]; /* Then print its ASCII equivalent */
+		i++;
+	} while (i < length && i%16 != 0);
+    /* Copy line to output buffer */
+    line2print[LINE_LEN-1]='\n';
+    memcpy(current_line, line2print, LINE_LEN);
+    current_line += LINE_LEN;
+  }
+  buffer[bytes2alloc-1]='\0'; 
+  return buffer;
+} /* End of hexdump() */
+
+/* This is like strtol or atoi, but it allows digits only. No whitespace, sign,
+   or radix prefix. */
+long parse_long(const char *s, char **tail)
+{
+    if (!isdigit((int) (unsigned char) *s)) {
+        *tail = (char *) s;
+        return 0;
+    }
+
+    return strtol(s, (char **) tail, 10);
+}
+
+
+
+/* This function takes a byte count and stores a short ascii equivalent
+   in the supplied buffer. Eg: 0.122MB, 10.322Kb or 128B. */
+char *format_bytecount(unsigned long long bytes, char *buf, size_t buflen) {
+  assert(buf != NULL);
+
+  if (bytes < 1000)
+    Snprintf(buf, buflen, "%uB", (unsigned int) bytes);
+  else if (bytes < 1000000)
+    Snprintf(buf, buflen, "%.3fKB", bytes / 1000.0);
+  else
+    Snprintf(buf, buflen, "%.3fMB", bytes / 1000000.0);
+
+  return buf;
+}
+
+/* Compare a canonical option name (e.g. "max-scan-delay") with a
+   user-generated option such as "max_scan_delay" and returns 0 if the
+   two values are considered equivalant (for example, - and _ are
+   considered to be the same), nonzero otherwise. */
+int optcmp(const char *a, const char *b) {
+  while(*a && *b) {
+    if (*a == '_' || *a == '-') {
+      if (*b != '_' && *b != '-')
+	return 1;
+    }
+    else if (*a != *b)
+      return 1;
+    a++; b++;
+  }
+  if (*a || *b)
+    return 1;
+  return 0;
+}
+
+/* Returns one if the file pathname given exists, is not a directory and
+ * is readable by the executing process.  Returns two if it is readable
+ * and is a directory.  Otherwise returns 0. */
+int fileexistsandisreadable(const char *pathname) {
+	char *pathname_buf = strdup(pathname);
+	int status = 0;
+	struct stat st;
+
+#ifdef WIN32
+	// stat on windows only works for "dir_name" not for "dir_name/" or "dir_name\\"
+	int pathname_len = strlen(pathname_buf);
+	char last_char = pathname_buf[pathname_len - 1];
+
+	if(	last_char == '/'
+		|| last_char == '\\')
+		pathname_buf[pathname_len - 1] = '\0';
+
+#endif
+
+  if (stat(pathname_buf, &st) == -1)
+    status = 0;
+  else if (access(pathname_buf, R_OK) != -1)
+    status = S_ISDIR(st.st_mode) ? 2 : 1;
+
+  free(pathname_buf);
+  return status;
+}
+
+#if HAVE_PROC_SELF_EXE
+static char *executable_path_proc_self_exe(void) {
+  char buf[1024];
+  char *path;
+  int n;
+
+  n = readlink("/proc/self/exe", buf, sizeof(buf));
+  if (n < 0 || n >= sizeof(buf))
+    return NULL;
+  path = (char *) safe_malloc(n + 1);
+  /* readlink does not null-terminate. */
+  memcpy(path, buf, n);
+  path[n] = '\0';
+
+  return path;
+}
+#endif
+
+#if HAVE_MACH_O_DYLD_H
+#include <mach-o/dyld.h>
+/* See the dyld(3) man page on OS X. */
+static char *executable_path_NSGetExecutablePath(void) {
+  char buf[1024];
+  uint32_t size;
+
+  size = sizeof(buf);
+  if (_NSGetExecutablePath(buf, &size) == 0)
+    return strdup(buf);
+  else
+    return NULL;
+}
+#endif
+
+#if WIN32
+static char *executable_path_GetModuleFileName(void) {
+  char buf[1024];
+  int n;
+
+  n = GetModuleFileName(GetModuleHandle(0), buf, sizeof(buf));
+  if (n <= 0 || n >= sizeof(buf))
+    return NULL;
+
+  return strdup(buf);
+}
+#endif
+
+static char *executable_path_argv0(const char *argv0) {
+  if (argv0 == NULL)
+    return NULL;
+  /* We can get the path from argv[0] if it contains a directory separator.
+     (Otherwise it was looked up in $PATH). */
+  if (strchr(argv0, '/') != NULL)
+    return strdup(argv0);
+#if WIN32
+  if (strchr(argv0, '\\') != NULL)
+    return strdup(argv0);
+#endif
+  return NULL;
+}
+
+char *executable_path(const char *argv0) {
+  char *path;
+
+  path = NULL;
+#if HAVE_PROC_SELF_EXE
+  if (path == NULL)
+    path = executable_path_proc_self_exe();
+#endif
+#if HAVE_MACH_O_DYLD_H
+  if (path == NULL)
+    path = executable_path_NSGetExecutablePath();
+#endif
+#if WIN32
+  if (path == NULL)
+    path = executable_path_GetModuleFileName();
+#endif
+  if (path == NULL)
+    path = executable_path_argv0(argv0);
+
+  return path;
+}
