@@ -1,4 +1,95 @@
-/* $Id: ncat_main.c 13816 2009-06-18 14:45:13Z david $ */
+/***************************************************************************
+ * ncat_main.c -- main function: option parsing and checking, dispatching  *
+ * to mode-specific functions.                                             *
+ ***********************IMPORTANT NMAP LICENSE TERMS************************
+ *                                                                         *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
+ *                                                                         *
+ * Note that the GPL places important restrictions on "derived works", yet *
+ * it does not provide a detailed definition of that term.  To avoid       *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
+ * "derivative work" for the purpose of this license if it does any of the *
+ * following:                                                              *
+ * o Integrates source code from Nmap                                      *
+ * o Reads or includes Nmap copyrighted data files, such as                *
+ *   nmap-os-db or nmap-service-probes.                                    *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                *
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
+ * o Links to a library or executes a program that does any of the above   *
+ *                                                                         *
+ * The term "Nmap" should be taken to also include any portions or derived *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
+ *                                                                         *
+ * If you have any questions about the GPL licensing restrictions on using *
+ * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
+ * we also offer alternative license to integrate Nmap into proprietary    *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
+ *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
+ * If you received these files with a written license agreement or         *
+ * contract stating terms other than the terms above, then that            *
+ * alternative license agreement takes precedence over these comments.     *
+ *                                                                         *
+ * Source is provided to this software because we believe users have a     *
+ * right to know exactly what a program is going to do before they run it. *
+ * This also allows you to audit the software for security holes (none     *
+ * have been found so far).                                                *
+ *                                                                         *
+ * Source code also allows you to port Nmap to new platforms, fix bugs,    *
+ * and add new features.  You are highly encouraged to send your changes   *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
+ * distribution.  By sending these changes to Fyodor or one of the         *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
+ *                                                                         *
+ * This program is distributed in the hope that it will be useful, but     *
+ * WITHOUT ANY WARRANTY; without even the implied warranty of              *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
+ *                                                                         *
+ ***************************************************************************/
+
+/* $Id: ncat_main.c 28195 2012-03-01 09:05:02Z henri $ */
 
 #include "nsock.h"
 #include "ncat.h"
@@ -46,7 +137,7 @@ static void parseproxy(char *str, struct sockaddr_storage *ss, unsigned short de
         portno = defport;
 
     if (!resolve(ptr, portno, ss, &sslen, o.af)) {
-        loguser("%s: Could not resolve proxy \"%s\".\n", NCAT_NAME, ptr);
+        loguser("Could not resolve proxy \"%s\".\n", ptr);
         if (o.af == AF_INET6 && httpproxy)
             loguser("Did you specify the port number? It's required for IPv6.\n");
         exit(EXIT_FAILURE);
@@ -100,7 +191,7 @@ static void host_list_to_set(struct addrset *set, struct host_list_node *list)
             fd = fopen(node->spec, "r");
             if (fd == NULL)
                 bye("can't open %s: %s.", node->spec, strerror(errno));
-            if (!addrset_add_file(set, fd))
+            if (!addrset_add_file(set, fd, o.af, !o.nodns))
                 bye("error in hosts file %s.", node->spec);
             fclose(fd);
         } else {
@@ -109,25 +200,20 @@ static void host_list_to_set(struct addrset *set, struct host_list_node *list)
             commalist = node->spec;
             while ((spec = strtok(commalist, ",")) != NULL) {
                 commalist = NULL;
-                if (!addrset_add_spec(set, spec))
+                if (!addrset_add_spec(set, spec, o.af, !o.nodns))
                     bye("error in host specification \"%s\".", node->spec);
             }
         }
     }
 }
 
-static int print_banner(FILE *fp)
+static void print_banner(void)
 {
-    return fprintf(fp, "%s version %s ( %s )\n", NCAT_NAME, NCAT_VERSION, NCAT_URL);
+    loguser("Version %s ( %s )\n", NCAT_VERSION, NCAT_URL);
 }
 
 int main(int argc, char *argv[])
 {
-    struct sockaddr_in *sin = (struct sockaddr_in *) &targetss;
-#ifdef HAVE_IPV6
-    static struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &targetss;
-#endif
-
     /* We have to buffer the lists of hosts to allow and deny until after option
        parsing is done. Adding hosts to an addrset can require name resolution,
        which may differ as a result of options like -n and -6. */
@@ -138,9 +224,6 @@ int main(int argc, char *argv[])
     char *source = NULL;
     char *proxyaddr = NULL;
 
-    /* Set default options. */
-    options_init();
-
     struct option long_options[] = {
         {"4",               no_argument,        NULL,         '4'},
         {"6",               no_argument,        NULL,         '6'},
@@ -155,13 +238,14 @@ int main(int argc, char *argv[])
         {"listen",          no_argument,        NULL,         'l'},
         {"output",          required_argument,  NULL,         'o'},
         {"hex-dump",        required_argument,  NULL,         'x'},
+        {"append-output",   no_argument,        NULL,         0},
         {"idle-timeout",    required_argument,  NULL,         'i'},
         {"keep-open",       no_argument,        NULL,         'k'},
         {"recv-only",       no_argument,        &o.recvonly,  1},
         {"source-port",     required_argument,  NULL,         'p'},
         {"source",          required_argument,  NULL,         's'},
         {"send-only",       no_argument,        &o.sendonly,  1},
-        {"broker",          no_argument,        &o.broker,    1},
+        {"broker",          no_argument,        NULL,         0},
         {"chat",            no_argument,        NULL,         0},
         {"talk",            no_argument,        NULL,         0},
         {"deny",            required_argument,  NULL,         0},
@@ -170,6 +254,7 @@ int main(int argc, char *argv[])
         {"allowfile",       required_argument,  NULL,         0},
         {"telnet",          no_argument,        NULL,         't'},
         {"udp",             no_argument,        NULL,         'u'},
+        {"sctp",            no_argument,        &o.sctp,      1},
         {"version",         no_argument,        NULL,         0},
         {"verbose",         no_argument,        NULL,         'v'},
         {"wait",            required_argument,  NULL,         'w'},
@@ -177,6 +262,7 @@ int main(int argc, char *argv[])
         {"proxy",           required_argument,  NULL,         0},
         {"proxy-type",      required_argument,  NULL,         0},
         {"proxy-auth",      required_argument,  NULL,         0},
+        {"nsock-engine",    required_argument,  NULL,         0},
 #ifdef HAVE_OPENSSL
         {"ssl",             no_argument,        &o.ssl,       1},
         {"ssl-cert",        required_argument,  NULL,         0},
@@ -187,6 +273,10 @@ int main(int argc, char *argv[])
         {0, 0, 0, 0}
     };
 
+    gettimeofday(&start_time, NULL);
+    /* Set default options. */
+    options_init();
+
 #ifdef WIN32
     windows_init();
 #endif
@@ -216,19 +306,20 @@ int main(int argc, char *argv[])
             o.crlf = 1;
             break;
         case 'c':
+            o.cmdexec = optarg;
             o.shellexec = 1;
-            /* fall through */
+	    break;
         case 'e':
             o.cmdexec = optarg;
             break;
         case 'g': {
             char *a = strtok(optarg, ",");
             do {
-                struct sockaddr_in addr;
+                union sockaddr_u addr;
                 size_t sslen;
-                if (!resolve(a, 0, (struct sockaddr_storage *) &addr, &sslen, AF_INET))
+                if (!resolve(a, 0, &addr.storage, &sslen, AF_INET))
                     bye("Sorry, could not resolve source route hop %s.", a);
-                o.srcrtes[o.numsrcrtes] = addr.sin_addr;
+                o.srcrtes[o.numsrcrtes] = addr.in.sin_addr;
             } while (o.numsrcrtes++ <= 8 && (a = strtok(NULL, ",")));
             if (o.numsrcrtes > 8)
                 bye("Sorry, you gave too many source route hops.");
@@ -249,12 +340,14 @@ int main(int argc, char *argv[])
             o.linedelay = tval2msecs(optarg);
             if (o.linedelay <= 0)
                 bye("Invalid -d delay (must be greater than 0).", optarg);
+            if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
+                bye("Since April 2010, the default unit for -d is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
             break;
         case 'o':
-            o.normlogfd = ncat_openlog(optarg);
+            o.normlog = optarg;
             break;
         case 'x':
-            o.hexlogfd = ncat_openlog(optarg);
+            o.hexlog = optarg;
             break;
         case 'p':
             srcport = atoi(optarg);
@@ -265,6 +358,8 @@ int main(int argc, char *argv[])
             o.idletimeout = tval2msecs(optarg);
             if (o.idletimeout <= 0)
                 bye("Invalid -i timeout (must be greater than 0).");
+            if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
+                bye("Since April 2010, the default unit for -i is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
             break;
         case 's':
             source = optarg;
@@ -289,13 +384,15 @@ int main(int argc, char *argv[])
             o.conntimeout = tval2msecs(optarg);
             if (o.conntimeout <= 0)
                 bye("Invalid -w timeout (must be greater than 0).");
+            if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
+                bye("Since April 2010, the default unit for -w is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
             break;
         case 't':
             o.telnet = 1;
             break;
         case 0:
             if (strcmp(long_options[option_index].name, "version") == 0) {
-                print_banner(stdout);
+                print_banner();
                 exit(EXIT_SUCCESS);
             }
             else if (strcmp(long_options[option_index].name, "proxy") == 0)
@@ -316,6 +413,16 @@ int main(int argc, char *argv[])
                         bye("You can't specify more than one --proxy-auth.");
                 o.proxy_auth = Strdup(optarg);
             }
+            else if (strcmp(long_options[option_index].name, "nsock-engine") == 0)
+            {
+                nsock_set_default_engine(optarg);
+            }
+            else if (strcmp(long_options[option_index].name, "broker") == 0)
+            {
+                o.broker = 1;
+                /* --broker implies --listen. */
+                o.listen = 1;
+            }
             else if (strcmp(long_options[option_index].name, "chat") == 0
                      || strcmp(long_options[option_index].name, "talk") == 0)
             {
@@ -342,13 +449,19 @@ int main(int argc, char *argv[])
             {
                 host_list_add_filename(&deny_host_list, optarg);
             }
+            else if (strcmp(long_options[option_index].name, "append-output") == 0)
+            {
+                o.append = 1;
+            }
 #ifdef HAVE_OPENSSL
             else if (strcmp(long_options[option_index].name, "ssl-cert") == 0)
             {
+                o.ssl = 1;
                 o.sslcert = Strdup(optarg);
             }
             else if (strcmp(long_options[option_index].name, "ssl-key") == 0)
             {
+                o.ssl = 1;
                 o.sslkey = Strdup(optarg);
             }
             else if (strcmp(long_options[option_index].name, "ssl-verify") == 0)
@@ -358,6 +471,7 @@ int main(int argc, char *argv[])
             }
             else if (strcmp(long_options[option_index].name, "ssl-trustfile") == 0)
             {
+                o.ssl = 1;
                 if (o.ssltrustfile != NULL)
                     bye("The --ssl-trustfile option may be given only once.");
                 o.ssltrustfile = Strdup(optarg);
@@ -372,37 +486,39 @@ int main(int argc, char *argv[])
             printf(
 "Usage: ncat [options] [hostname] [port]\n"
 "\n"
-"Options taking a time assume milliseconds, unless you append an 's'\n"
-"(seconds), 'm' (minutes), or 'h' (hours) to the value (e.g. 30s)\n"
+"Options taking a time assume seconds. Append 'ms' for milliseconds,\n"
+"'s' for seconds, 'm' for minutes, or 'h' for hours (e.g. 500ms).\n"
 "  -4                         Use IPv4 only\n"
 "  -6                         Use IPv6 only\n"
 "  -C, --crlf                 Use CRLF for EOL sequence\n"
-"  -c, --sh-exec <command>    Executes specified command via /bin/sh\n"
-"  -e, --exec <command>       Executes specified command\n"
+"  -c, --sh-exec <command>    Executes the given command via /bin/sh\n"
+"  -e, --exec <command>       Executes the given command\n"
 "  -g hop1[,hop2,...]         Loose source routing hop points (8 max)\n"
-"  -G n                       Loose source routing hop pointer (4, 8, 12, ...)\n"
-"  -m, --max-conns n          Maximum n simultaneous connections\n"
+"  -G <n>                     Loose source routing hop pointer (4, 8, 12, ...)\n"
+"  -m, --max-conns <n>        Maximum <n> simultaneous connections\n"
 "  -h, --help                 Display this help screen\n"
 "  -d, --delay <time>         Wait between read/writes\n"
-"  -o, --output               Dump session data to a file\n"
-"  -x, --hex-dump             Dump session data as hex to a file\n"
+"  -o, --output <filename>    Dump session data to a file\n"
+"  -x, --hex-dump <filename>  Dump session data as hex to a file\n"
 "  -i, --idle-timeout <time>  Idle read/write timeout\n"
-"  -p, --source-port port     Specify source port to use (doesn't affect -l)\n"
+"  -p, --source-port port     Specify source port to use\n"
 "  -s, --source addr          Specify source address to use (doesn't affect -l)\n"
 "  -l, --listen               Bind and listen for incoming connections\n"
 "  -k, --keep-open            Accept multiple connections in listen mode\n"
 "  -n, --nodns                Do not resolve hostnames via DNS\n"
 "  -t, --telnet               Answer Telnet negotiations\n"
 "  -u, --udp                  Use UDP instead of default TCP\n"
+"      --sctp                 Use SCTP instead of default TCP\n"
 "  -v, --verbose              Set verbosity level (can be used up to 3 times)\n"
 "  -w, --wait <time>          Connect timeout\n"
+"      --append-output        Append rather than clobber specified output files\n"
 "      --send-only            Only send data, ignoring received; quit on EOF\n"
 "      --recv-only            Only receive data, never send anything\n"
-"      --allow                Allow specific hosts to connect to Ncat\n"
+"      --allow                Allow only given hosts to connect to Ncat\n"
 "      --allowfile            A file of hosts allowed to connect to Ncat\n"
-"      --deny                 Hosts to be denied from connecting to Ncat\n"
+"      --deny                 Deny given hosts from connecting to Ncat\n"
 "      --denyfile             A file of hosts denied from connecting to Ncat\n"
-"      --broker               Enable Ncat's Connection Brokering mode\n"
+"      --broker               Enable Ncat's connection brokering mode\n"
 "      --chat                 Start a simple Ncat chat server\n"
 "      --proxy <addr[:port]>  Specify address of host to proxy through\n"
 "      --proxy-type <type>    Specify proxy type (\"http\" or \"socks4\")\n"
@@ -428,13 +544,29 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (o.normlog)
+        o.normlogfd = ncat_openlog(o.normlog, o.append);
+    if (o.hexlog)
+        o.hexlogfd = ncat_openlog(o.hexlog, o.append);
+
     if (o.verbose)
-        print_banner(stderr);
+        print_banner();
+
+    if (o.debug)
+        nbase_set_log(loguser, logdebug);
+    else
+        nbase_set_log(loguser, NULL);
 
     /* Will be AF_INET or AF_INET6 when valid */
-    memset(&targetss, 0, sizeof(targetss));
-    targetss.ss_family = AF_UNSPEC;
-    httpconnect = socksconnect = srcaddr = targetss;
+    memset(&targetss.storage, 0, sizeof(targetss.storage));
+    targetss.storage.ss_family = AF_UNSPEC;
+    httpconnect.storage = socksconnect.storage = srcaddr.storage = targetss.storage;
+
+    /* Clear the listenaddrs array */
+    int i;
+    for (i = 0; i < NUM_LISTEN_ADDRS; i++) {
+        listenaddrs[i].storage = targetss.storage;
+    }
 
     if (proxyaddr) {
       if (!o.proxytype)
@@ -447,11 +579,11 @@ int main(int argc, char *argv[])
            * colons in the IPv6 address and host:port separator).
            */
 
-          parseproxy(proxyaddr, &httpconnect, DEFAULT_PROXY_PORT);
+          parseproxy(proxyaddr, &httpconnect.storage, DEFAULT_PROXY_PORT);
       } else if (!strcmp(o.proxytype, "socks4") || !strcmp(o.proxytype, "4")) {
           /* Parse SOCKS proxy address and temporarily store it in socksconnect */
 
-          parseproxy(proxyaddr, &socksconnect, DEFAULT_SOCKS4_PORT);
+          parseproxy(proxyaddr, &socksconnect.storage, DEFAULT_SOCKS4_PORT);
       } else {
           bye("Invalid proxy type \"%s\".", o.proxytype);
       }
@@ -467,35 +599,18 @@ int main(int argc, char *argv[])
       }
     }
 
+    /* Default port */
+    o.portno = DEFAULT_NCAT_PORT;
+
     /* Resolve the given source address */
     if (source) {
         if (o.listen)
             bye("-l and -s are incompatible.  Specify the address and port to bind to like you would a host to connect to.");
 
-        if (!resolve(source, 0, &srcaddr, &srcaddrlen, o.af))
+        if (!resolve(source, 0, &srcaddr.storage, &srcaddrlen, o.af))
             bye("Could not resolve source address %s.", source);
     }
 
-    if (srcport != -1) {
-        if (o.listen)
-            bye("-l and -p are incompatible.  Specify the address and port to bind to like you would a host to connect to.");
-
-        if (srcaddr.ss_family == AF_UNSPEC)
-            srcaddr.ss_family = o.af;
-        if (o.af == AF_INET) {
-            ((struct sockaddr_in *) &srcaddr)->sin_port = htons((unsigned short) srcport);
-            if (!srcaddrlen)
-                srcaddrlen = sizeof(struct sockaddr_in);
-        }
-#ifdef HAVE_IPV6
-        else {
-            ((struct sockaddr_in6 *) &srcaddr)->sin6_port = htons((unsigned short) srcport);
-            if (!srcaddrlen)
-                srcaddrlen = sizeof(struct sockaddr_in6);
-        }
-#endif
-    }
-
     host_list_to_set(&o.allowset, allow_host_list);
     host_list_free(allow_host_list);
     host_list_to_set(&o.denyset, deny_host_list);
@@ -510,7 +625,7 @@ int main(int argc, char *argv[])
         if (strspn(argv[optind], "0123456789") != strlen(argv[optind])) {
             o.target = argv[optind];
             /* resolve hostname */
-            if (!resolve(o.target, 0, &targetss, &targetsslen, o.af))
+            if (!resolve(o.target, 0, &targetss.storage, &targetsslen, o.af))
                 bye("Could not resolve hostname %s.", o.target);
             optind++;
         } else {
@@ -519,34 +634,67 @@ int main(int argc, char *argv[])
         }
     }
 
-    if (optind < argc) {
-        long long_port = Strtol(argv[optind], NULL, 10);
+    /* Whatever's left is the port number; there should be at most one. */
+    if (optind + 1 < argc || (o.listen && srcport != -1 && optind + 1 == argc)) {
+        loguser("Got more than one port specification:");
+        if (o.listen && srcport != -1)
+            loguser_noprefix(" %d", srcport);
+        for (; optind < argc; optind++)
+            loguser_noprefix(" %s", argv[optind]);
+        loguser_noprefix(". QUITTING.\n");
+        exit(2);
+    } else if (optind + 1 == argc) {
+        long long_port;
 
-        if (long_port <= 0 || long_port > 65535)
-            bye("Invalid port number.");
+        errno = 0;
+        long_port = strtol(argv[optind], NULL, 10);
+        if (errno != 0 || long_port <= 0 || long_port > 65535)
+            bye("Invalid port number \"%s\".", argv[optind]);
 
         o.portno = (unsigned short) long_port;
-    } else {
-        /* Default port */
-        o.portno = DEFAULT_NCAT_PORT;
     }
 
     if (o.af == AF_INET)
-        sin->sin_port = htons(o.portno);
+        targetss.in.sin_port = htons(o.portno);
 #ifdef HAVE_IPV6
     else
-        sin6->sin6_port = htons(o.portno);
+        targetss.in6.sin6_port = htons(o.portno);
 #endif
 
+    if (srcport != -1) {
+        if (o.listen) {
+            /* Treat "ncat -l -p <port>" the same as "ncat -l <port>" for nc
+               compatibility. */
+            o.portno = srcport;
+        } else {
+            if (srcaddr.storage.ss_family == AF_UNSPEC) {
+                /* We have a source port but not an explicit source address;
+                   fill in an unspecified address of the same family as the
+                   target. */
+                srcaddr.storage.ss_family = targetss.storage.ss_family;
+                if (srcaddr.storage.ss_family == AF_INET)
+                    srcaddr.in.sin_addr.s_addr = INADDR_ANY;
+                else if (srcaddr.storage.ss_family == AF_INET6)
+                    srcaddr.in6.sin6_addr = in6addr_any;
+            }
+            if (srcaddr.storage.ss_family == AF_INET)
+                srcaddr.in.sin_port = htons(srcport);
+#ifdef HAVE_IPV6
+            else
+                srcaddr.in6.sin6_port = htons(srcport);
+#endif
+        }
+    }
+
     /* Since the host we're actually *connecting* to is the proxy server, we
      * need to reverse these address structures to avoid any further confusion
      */
-    if (httpconnect.ss_family != AF_UNSPEC) {
-        struct sockaddr_storage tmp = targetss;
+    if (httpconnect.storage.ss_family != AF_UNSPEC) {
+        union sockaddr_u tmp = targetss;
         targetss = httpconnect;
         httpconnect = tmp;
-    } else if (socksconnect.ss_family != AF_UNSPEC) {
-        struct sockaddr_storage tmp = targetss;
+    } else if (socksconnect.storage.ss_family != AF_UNSPEC) {
+        union sockaddr_u tmp = targetss;
         targetss = socksconnect;
         socksconnect = tmp;
     }
@@ -555,6 +703,8 @@ int main(int argc, char *argv[])
         /* Don't allow a false sense of security if someone tries SSL over UDP. */
         if (o.ssl)
             bye("UDP mode does not support SSL.");
+        if (o.keepopen && o.cmdexec == NULL)
+            bye("UDP mode does not support the -k or --keep-open options, except with --exec or --sh-exec.");
         if (o.broker)
             bye("UDP mode does not support connection brokering.\n\
 If this feature is important to you, write nmap-dev@insecure.org with a\n\
@@ -562,24 +712,6 @@ description of how you intend to use it, as an aid to deciding how UDP\n\
 connection brokering should work.");
     }
 
-    /*
-     * Ncat obviously just redirects stdin/stdout/stderr, so have no clue what is going on with
-     * the underlying application. If it foobars, it's your problem.
-     */
-    if (o.cmdexec && o.normlogfd != -1)
-        bye("Invalid option combination: `-e' and `-o'.");
-
-    /* ditto. */
-    if (o.cmdexec && o.hexlogfd != -1)
-        bye("Invalid option combination: `-e' and `-x'.");
-
-    /* This doesn't work because we just fork and redirect the child's standard
-       input and output to a socket. To support --exec with --ssl we would need
-       to fork Ncat code to speak SSL to the socket and plaintext to the
-       subprocess. */
-    if (o.cmdexec && o.ssl)
-        bye("Invalid option combination: `-e' and `--ssl'.");
-
     /* Do whatever is necessary to receive \n for line endings on input from
        the console. A no-op on Unix. */
     set_lf_mode();
@@ -603,9 +735,8 @@ static int ncat_connect_mode(void) {
     if (o.conn_limit != -1)
         bye("Invalid option combination: `--max-conns' with connect.");
 
-    /* connection brokering only applies in listen mode. */
-    if (o.broker)
-        bye("Invalid option combination: `--broker' with connect.");
+    if (o.chat)
+        bye("Invalid option combination: `--chat' with connect.");
 
     if (o.keepopen)
         bye("Invalid option combination: `--keep-open' with connect.");
@@ -614,25 +745,23 @@ static int ncat_connect_mode(void) {
 }
 
 static int ncat_listen_mode(void) {
-    struct sockaddr_in *sin = (struct sockaddr_in *) &targetss;
-#ifdef HAVE_IPV6
-    static struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &targetss;
-#endif
-
     /* Can't 'listen' AND 'connect' to a proxy server at the same time. */
-    if (httpconnect.ss_family != AF_UNSPEC || socksconnect.ss_family != AF_UNSPEC)
+    if (httpconnect.storage.ss_family != AF_UNSPEC || socksconnect.storage.ss_family != AF_UNSPEC)
         bye("Invalid option combination: --proxy and -l.");
 
     if (o.idletimeout != 0)
         bye("An idle timeout only works in connect mode.");
 
-    /* If a non-root user tries to bind to a privileged port, Exit. */
-    if (o.portno < 1024 && !ncat_checkuid())
-        bye("Attempted a non-root bind() to a port <1024.");
-
     if (o.broker && o.cmdexec != NULL)
         bye("Invalid option combination: --broker and -e.");
 
+    if (o.proxytype != NULL && o.telnet)
+        bye("Invalid option combination: --telnet has no effect with --proxy-type.");
+
+    if (o.conn_limit != -1 && !(o.keepopen || o.broker))
+        loguser("Warning: Maximum connections ignored, since it does not take "
+                "effect without -k or --broker.\n");
+
     /* Set the default maximum simultaneous TCP connection limit. */
     if (o.conn_limit == -1)
         o.conn_limit = DEFAULT_MAX_CONNS;
@@ -643,34 +772,33 @@ static int ncat_listen_mode(void) {
         bye("/bin/sh is not executable, so `-c' won't work.");
 #endif
 
-    /* If we weren't given a specific address to listen on, we accept
-     * any incoming connections
-     */
-    if (targetss.ss_family == AF_UNSPEC) {
-        targetss.ss_family = o.af;
+    if (targetss.storage.ss_family != AF_UNSPEC) {
+        listenaddrs[num_listenaddrs++] = targetss;
+    } else {
+        size_t ss_len;
+        int rc;
 
-        if (o.af == AF_INET)
-            sin->sin_addr.s_addr = INADDR_ANY;
+        /* No command-line address. Listen on IPv4 or IPv6 or both. */
+	/* Try to bind to IPv6 first; on AIX a bound IPv4 socket blocks an IPv6
+           socket on the same port, despite IPV6_V6ONLY. */
 #ifdef HAVE_IPV6
-        else
-            sin6->sin6_addr = in6addr_any;
+        if (o.af == AF_INET6 || o.af == AF_UNSPEC) {
+            ss_len = sizeof(listenaddrs[num_listenaddrs]);
+            rc = resolve("::", o.portno, &listenaddrs[num_listenaddrs].storage, &ss_len, AF_INET6);
+            if (!rc)
+                bye("Failed to resolve default IPv6 address.");
+            num_listenaddrs++;
+        }
 #endif
+        if (o.af == AF_INET || o.af == AF_UNSPEC) {
+            ss_len = sizeof(listenaddrs[num_listenaddrs]);
+            rc = resolve("0.0.0.0", o.portno, &listenaddrs[num_listenaddrs].storage, &ss_len, AF_INET);
+            if (!rc)
+                bye("Failed to resolve default IPv4 address.");
+            num_listenaddrs++;
+        }
     }
 
-    /* Actually set our source address */
-    srcaddr = targetss;
-
-    if (srcaddr.ss_family == AF_INET)
-        srcaddrlen = sizeof(struct sockaddr_in);
-#ifdef HAVE_IPV6
-    else
-        srcaddrlen = sizeof(struct sockaddr_in6);
-#endif
-
-    /* If --broker was supplied, go into connection brokering mode. */
-    if (o.broker)
-        return ncat_broker();
-
     if (o.proxytype) {
         if (strcmp(o.proxytype, "http") == 0)
             o.httpserver = 1;
