@@ -2,7 +2,7 @@
  * sys_wrap.c -- Error-checked wrappers around common functions.           *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -12,11 +12,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -24,31 +25,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -66,15 +68,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -85,25 +88,14 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: sys_wrap.c 15803 2009-10-10 02:42:28Z david $ */
+/* $Id: sys_wrap.c 28192 2012-03-01 06:53:35Z fyodor $ */
 
 #include <limits.h>
 
 #include "sys_wrap.h"
 #include "util.h"
 
-int Bind(int sockfd, const struct sockaddr *my_addr, socklen_t addrlen)
-{
-    int ret;
-
-    ret = bind(sockfd, my_addr, addrlen);
-    if(ret < 0)
-        die("bind");
-
-    return ret;
-}
-
-void * Calloc(size_t nmemb, size_t size)
+void *Calloc(size_t nmemb, size_t size)
 {
     void    *ret;
 
@@ -111,7 +103,7 @@ void * Calloc(size_t nmemb, size_t size)
     smul(nmemb, size);
 
     ret = calloc(nmemb, size);
-    if(ret == NULL)
+    if (ret == NULL)
         die("calloc");
 
     return ret;
@@ -119,7 +111,7 @@ void * Calloc(size_t nmemb, size_t size)
 
 int Close(int fd)
 {
-    if(close(fd) < 0)
+    if (close(fd) < 0)
         die("close");
 
     return 0;
@@ -127,7 +119,7 @@ int Close(int fd)
 
 int Connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
 {
-    if(connect(sockfd, serv_addr, addrlen) < 0)
+    if (connect(sockfd, serv_addr, addrlen) < 0)
         die("connect");
 
     return 0;
@@ -138,7 +130,7 @@ int Dup2(int oldfd, int newfd)
     int ret;
 
     ret = dup2(oldfd, newfd);
-    if(ret < 0)
+    if (ret < 0)
         die("dup2");
 
     return ret;
@@ -146,7 +138,7 @@ int Dup2(int oldfd, int newfd)
 
 int Listen(int s, int backlog)
 {
-    if(listen(s, backlog) < 0)
+    if (listen(s, backlog) < 0)
         die("listen");
 
     return 0;
@@ -157,7 +149,7 @@ int Open(const char *pathname, int flags, mode_t mode)
     int ret;
 
     ret = open(pathname, flags, mode);
-    if(ret < 0)
+    if (ret < 0)
         die("open");
 
     return ret;
@@ -168,7 +160,7 @@ ssize_t Read(int fd, void *buf, size_t count)
     ssize_t ret;
 
     ret = read(fd, buf, count);
-    if(ret < 0)
+    if (ret < 0)
         die("read");
 
     return ret;
@@ -179,7 +171,7 @@ ssize_t Recv(int s, void *buf, size_t len, int flags)
     ssize_t ret;
 
     ret = recv(s, (char*)buf, len, flags);
-    if(ret < 0)
+    if (ret < 0)
         die("recv");
 
     return ret;
@@ -191,7 +183,7 @@ ssize_t Recvfrom(int s, void *buf, size_t len, int flags,
     ssize_t ret;
 
     ret = recvfrom(s, (char*)buf, len, flags, from, fromlen);
-    if(ret < 0)
+    if (ret < 0)
         die("recvfrom");
 
     return ret;
@@ -203,7 +195,7 @@ int Setsockopt(int s, int level, int optname, const void *optval,
     int ret;
 
     ret = setsockopt(s, level, optname, (const char*)optval, optlen);
-    if(ret < 0)
+    if (ret < 0)
         die("setsockopt");
 
     return ret;
@@ -214,7 +206,7 @@ sighandler_t Signal(int signum, sighandler_t handler)
     sighandler_t    ret;
 
     ret = signal(signum, handler);
-    if(ret == SIG_ERR)
+    if (ret == SIG_ERR)
         die("signal");
 
     return ret;
@@ -226,18 +218,18 @@ int Socket(int domain, int type, int protocol)
     int ret;
 
     ret = socket(domain, type, protocol);
-    if(ret < 0)
+    if (ret < 0)
         die("socket");
 
     return ret;
 }
 
-char * Strdup(const char *s)
+char *Strdup(const char *s)
 {
     char    *ret;
 
     ret = strdup(s);
-    if(ret == NULL)
+    if (ret == NULL)
         die("strdup");
 
     return ret;
@@ -247,7 +239,7 @@ ssize_t Write(int fd, const void *buf, size_t count)
 {
     ssize_t ret = write(fd, buf, count);
 
-    if(ret < 0)         /* we don't bail if < count bytes written */
+    if (ret < 0)         /* we don't bail if < count bytes written */
         die("write");
 
     return ret;
