@@ -182,14 +182,12 @@ static struct lstr *check(SSL *ssl, const struct lstr names[])
         if (ssl_post_connect_check(ssl, name->s)) {
             if (size >= capacity) {
                 capacity = (size + 1) * 2;
-                results = realloc(results, (capacity + 1) * sizeof(results[0]));
-                assert(results != NULL);
+                results = safe_realloc(results, (capacity + 1) * sizeof(results[0]));
             }
             results[size++] = *name;
         }
     }
-    results = realloc(results, (size + 1) * sizeof(results[0]));
-    assert(results != NULL);
+    results = safe_realloc(results, (size + 1) * sizeof(results[0]));
     results[size] = lstr_sentinel;
 
     return results;
@@ -361,12 +359,13 @@ err:
 
 static void print_escaped(const char *s, size_t len)
 {
+    int c;
     for ( ; len > 0; len--) {
-        if (isprint(*s) && !isspace(*s))
-            putchar(*s);
+        c = (unsigned char) *s++;
+        if (isprint(c) && !isspace(c))
+            putchar(c);
         else
-            printf("\\%03o", *s);
-        s++;
+            printf("\\%03o", c);
     }
 }
 
