@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -14,11 +14,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -32,19 +33,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -68,15 +70,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -96,6 +99,7 @@
 #include "output.h"
 #include "NpingOps.h"
 #include "ProbeMode.h"
+#include <signal.h>
 
 extern NpingOps o;
 extern EchoServer es;
@@ -413,16 +417,16 @@ clientid_t EchoServer::nep_match_headers(IPv4Header *ip4, IPv6Header *ip6, TCPHe
                     case PSPEC_TCP_SPORT:
                         if(tcp==NULL)break;
                         outPrint(DBG_3, "%s() Trying to match TCP Source Port", __func__);
-                        if( tcp->getSrcPort()==ntohs( *((u16 *)fspec->value) ) ){
-                            outPrint(DBG_3, "[Match] TCP Src=%u", tcp->getSrcPort());
+                        if( tcp->getSourcePort()==ntohs( *((u16 *)fspec->value) ) ){
+                            outPrint(DBG_3, "[Match] TCP Src=%u", tcp->getSourcePort());
                             current_score += 2 * FACTOR_TCP_SPORT;
                         }
                     break;
                     case PSPEC_TCP_DPORT:
                         if(tcp==NULL)break;
                         outPrint(DBG_3, "%s() Trying to match TCP Destination Port", __func__);
-                        if( tcp->getDstPort()==ntohs( *((u16 *)fspec->value) ) ){
-                            outPrint(DBG_3, "[Match] TCP Dst=%u", tcp->getDstPort());
+                        if( tcp->getDestinationPort()==ntohs( *((u16 *)fspec->value) ) ){
+                            outPrint(DBG_3, "[Match] TCP Dst=%u", tcp->getDestinationPort());
                             current_score += 2 * FACTOR_TCP_DPORT;
                         }
                     break;
@@ -485,16 +489,16 @@ clientid_t EchoServer::nep_match_headers(IPv4Header *ip4, IPv6Header *ip6, TCPHe
                     case PSPEC_UDP_SPORT:
                         if(udp==NULL)break;
                         outPrint(DBG_3, "%s() Trying to match UDP Source Port", __func__);
-                        if( udp->getSrcPort()==ntohs( *((u16 *)fspec->value) ) ){
-                            outPrint(DBG_3, "[Match] UDP Src=%u", udp->getSrcPort());
+                        if( udp->getSourcePort()==ntohs( *((u16 *)fspec->value) ) ){
+                            outPrint(DBG_3, "[Match] UDP Src=%u", udp->getSourcePort());
                             current_score += 2 * FACTOR_UDP_SPORT;
                         }
                     break;
                     case PSPEC_UDP_DPORT:
                         if(udp==NULL)break;
                         outPrint(DBG_3, "%s() Trying to match UDP Destination Port", __func__);
-                        if( udp->getDstPort()==ntohs( *((u16 *)fspec->value) ) ){
-                            outPrint(DBG_3, "[Match] UDP Dst=%u", udp->getDstPort());
+                        if( udp->getDestinationPort()==ntohs( *((u16 *)fspec->value) ) ){
+                            outPrint(DBG_3, "[Match] UDP Dst=%u", udp->getDestinationPort());
                             current_score += 2 * FACTOR_UDP_DPORT;
                         }
                     break;
@@ -1128,8 +1132,9 @@ int EchoServer::parse_hs_client(u8 *pkt, size_t pktlen, NEPContext *ctx){
 
   outPrint(DBG_3,"Session Key MAC_C2S:"); print_hexdump(DBG_3,ctx->getMacKeyC2S(), MAC_KEY_LEN);
   outPrint(DBG_3,"Session Key MAC_S2C:"); print_hexdump(DBG_3,ctx->getMacKeyS2C(), MAC_KEY_LEN);
-  outPrint(DBG_3,"Session Key CIPHER_C2S:"); print_hexdump(DBG_3,ctx->getCipherKeyS2C(), MAC_KEY_LEN);
-  outPrint(DBG_3,"Session Key CIPHER_S2C:"); print_hexdump(DBG_3,ctx->getCipherKeyC2S(), MAC_KEY_LEN);
+  outPrint(DBG_3,"Session Key CIPHER_C2S:"); print_hexdump(DBG_3,ctx->getCipherKeyC2S(), MAC_KEY_LEN);
+  outPrint(DBG_3,"Session Key CIPHER_S2C:"); print_hexdump(DBG_3,ctx->getCipherKeyS2C(), MAC_KEY_LEN);
+  
 
   /* Decrypt the encrypted part of the message before validating the MAC */
   if((next_iv=h.decrypt(ctx->getCipherKeyC2S(), CIPHER_KEY_LEN, ctx->getClientNonce(), TYPE_NEP_HANDSHAKE_CLIENT))==NULL){
@@ -1222,8 +1227,16 @@ int EchoServer::parse_packet_spec(u8 *pkt, size_t pktlen, NEPContext *ctx){
     if( h.getNextFieldSpec(&field, specbuff, &len)==OP_FAILURE ){
           break;
     }else{
-        ctx->addClientFieldSpec(field, len, specbuff);
-        recvspecs++;
+
+        /* Ensure the field spec is unique. Malicious users could try to supply
+         * the same spec more than once in order to get higher packet scores. */
+        if( ctx->isDuplicateFieldSpec(field) ){
+          outPrint(DBG_1, "Detected duplicate field specs in NEP_PACKET_SPEC message" );
+          return OP_FAILURE;
+        }else{
+            ctx->addClientFieldSpec(field, len, specbuff);
+            recvspecs++;
+        }
         /* Set a flag to indicate that mandatory IPv4 ID or IPv6 Flow has been
          * supplied by the client */
         if(h.getIPVersion()==0x04 && field==PSPEC_IPv4_ID)
@@ -1340,7 +1353,37 @@ int EchoServer::generate_echo(EchoHeader *h, const u8 *pkt, size_t pktlen, NEPCo
   h->setSequenceNumber( ctx->getNextServerSequence() );
   h->setTimestamp();
   h->setDLT(DLT_NODATALINKHEADERINCLUDED);
-  h->setEchoedPacket(pkt, pktlen);
+
+  /* If allowed, echo the whole packet, including any application layer data */
+  if( o.echoPayload() ){
+      h->setEchoedPacket(pkt, pktlen);
+  /* Otherwise, find if the packet contains application layer data and erase it */
+  }else{
+    /* Determine where the application data starts */
+    int offset=PacketParser::payload_offset(pkt, pktlen, false);
+    
+    /* If the packet does not have application data, don't touch it */
+    if(offset==0){
+        outPrint(DBG_3, "No payload found. Echoing the whole packet\n");
+        h->setEchoedPacket(pkt, pktlen);
+    /* If we found application data, zero it */
+    }else{
+        outPrint(DBG_3, "Erasing %d payload bytes\n", (int)pktlen-offset);
+        /* Allocate a new buffer, big enough to hold the packet */
+        u8 *new_pkt=(u8 *)safe_zalloc(pktlen);
+        /* Copy the initial header, and leave the rest as 0x00 bytes */
+        if(offset>0 && offset<(int)pktlen){
+            memcpy(new_pkt, pkt, offset);
+        /* If there was some error trying to find application data, include a
+         * default amount of data */
+        }else{
+            memcpy(new_pkt, pkt, MIN(pktlen, PAYLOAD_ECHO_BYTES_IN_DOUBT));
+        }
+        h->setEchoedPacket(new_pkt, pktlen);
+        free(new_pkt);
+    }
+  }
+
   h->setTotalLength();
   h->setMessageAuthenticationCode(ctx->getMacKeyS2C(), MAC_KEY_LEN);
 
@@ -1429,7 +1472,7 @@ int EchoServer::start() {
             ctx.generateServerNonce();
             ctx.generateInitialServerSequence();
             ctx.generateMacKeyS2CInitial();
-            outPrint(DBG_3,"Session Key MAC_C2S_INITIAL:"); print_hexdump(DBG_3,ctx.getMacKeyS2C(), MAC_KEY_LEN);
+            outPrint(DBG_3,"Session Key MAC_S2C_INITIAL:"); print_hexdump(DBG_3,ctx.getMacKeyS2C(), MAC_KEY_LEN);
 
             /* Craft NEP_HANDSHAKE_SERVER message */
             if( this->generate_hs_server(&h, &ctx)!=OP_SUCCESS)
@@ -1446,7 +1489,7 @@ int EchoServer::start() {
         }
         block_socket(listen_sd);
     }
-    /* Start the nsock loop. This never returns. Server quits on CTRL-C */
+    /* Sleep for a second until we check again for incoming connection requests */
     nsock_timer_create(nsp, empty_handler, 1000, NULL);
     loopret=nsock_loop(nsp, 1000);
   }
