@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -14,11 +14,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -32,19 +33,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -68,15 +70,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -103,6 +106,7 @@
 #include <sys/stat.h>
 
 #include "../libnetutil/netutil.h"
+#include "../libnetutil/npacket.h"
 
 #ifdef HAVE_CONFIG_H
     #include "nping_config.h"
@@ -285,7 +289,6 @@
 
 
 #define MAX_IP_PACKET_LEN 65535   /**< Max len of an IP datagram             */
-#define MAX_TCP_PAYLOAD_LEN 65495 /**< Max len of a TCP packet               */
 #define MAX_UDP_PAYLOAD_LEN 65507 /**< Check comments in UDPHeader::setSum() */
 
 #define MAX_DEV_LEN 128           /**< Max network interface name length     */
@@ -306,7 +309,8 @@
 /* General tunable defines  **************************************************/
 #define NPING_NAME "Nping"
 #define NPING_URL "http://nmap.org/nping"
-#define NPING_VERSION "0.5.50"
+#define NPING_VERSION "0.6.00"
+
 
 #define DEFAULT_VERBOSITY VB_0
 #define DEFAULT_DEBUGGING DBG_0
@@ -347,9 +351,12 @@
 /**< MTU used when user just supplies option -f but no MTU value */
 #define DEFAULT_MTU_FOR_FRAGMENTATION 72   
 
-#define DEFAULT_ICMP_TYPE 8  /**< Defualt ICMP message: Echo Request         */
+#define DEFAULT_ICMP_TYPE 8  /**< Default ICMP message: Echo Request         */
 #define DEFAULT_ICMP_CODE 0  /**< Default ICMP code: 0 (standard)            */
 
+#define DEFAULT_ICMPv6_TYPE 128 /**< Default ICMPv6 message: Echo Request    */
+#define DEFAULT_ICMPv6_CODE 0   /**< Default ICMPv6 code: 0 (standard)       */
+
 #define DEFAULT_ARP_OP 1   /**< Default ARP operation: OP_ARP_REQUEST      */
 
 /* WARNING: This is the max length for UDP and TCP payloads. Whatever you set
@@ -377,6 +384,22 @@
 /* (9929 because is prime as has not been assigned by IANA yet) */
 #define DEFAULT_ECHO_PORT 9929
 
+/* The echo server tries to zero any application layer data before echoing
+ * network packets. However, sometimes we may not be able to successfully
+ * parse a given packet (decide whether the packet contains application data
+ * or not), so this define specifies the amount of bytes of a packet that the
+ * server does not zero in such case. 40 bytes allows IPv4+TCP, an IPv6 header,
+ * an IPv4+UDP+12payload bytes, etc. In the case of UDP, the first 12 data bytes
+ * would be leaked. However, we should be able to parse simple IPv4-UDP packets
+ * without problem, so it should never happen. We expect to use this constant
+ * when received packets are really weird (eg. tunneled traffic, protocols we
+ * don't understand, etc. The 40 bytes are a compromise between dropping the
+ * packet but provide total protection against data leakeage due to attacks to
+ * the echo server, and providing some flexibility at the risk of leaking
+ * a few bytes if an attacker is able to trick the echo server into echoing
+ * packets that were not originated by him. */
+#define PAYLOAD_ECHO_BYTES_IN_DOUBT 40
+
 #define NSOCK_INFINITE -1
     
 /* Prototypes for nping.cc shared functions */
