@@ -1,254 +1,96 @@
---- SNMP functions.
+---
+-- SNMP functions.
 --
 -- @args snmpcommunity The community string to use. If not given, it is
 -- <code>"public"</code>, or whatever is passed to <code>buildPacket</code>.
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
 
-
 module(... or "snmp",package.seeall)
 
 
 require("bit")
+require("asn1")
 
----
--- Encodes an Integer according to ASN.1 basic encoding rules.
--- @param val Value to be encoded.
--- @return Encoded integer.
-local function encodeInt(val)
-   local lsb = 0
-   if val > 0 then
-      local valStr = ""
-      while (val > 0) do
-	 lsb = math.mod(val, 256)
-	 valStr = valStr .. bin.pack("C", lsb)
-	 val = math.floor(val/256)
-      end
-      if lsb > 127 then -- two's complement collision
-	 valStr = valStr .. bin.pack("H", "00")
-      end
+-- SNMP ASN.1 Encoders
+local tagEncoder = {}
 
-      return string.reverse(valStr)
-   elseif val < 0 then
-      local i = 1
-      local tcval = val + 256 -- two's complement
-      while tcval <= 127 do
-	 tcval = tcval + (math.pow(256, i) * 255)
-	 i = i+1
-      end
-      local valStr = ""
-      while (tcval > 0) do
-	 lsb = math.mod(tcval, 256)
-	 valStr = valStr .. bin.pack("C", lsb)
-	 tcval = math.floor(tcval/256)
-      end
-      return string.reverse(valStr)
-   else -- val == 0
-      return bin.pack("x")
-   end
+-- Override the boolean encoder
+tagEncoder['boolean'] = function(self, val)
+	return bin.pack('H', '05 00')
 end
 
+-- Complex tag encoders
+tagEncoder['table'] = function(self, val)
+	if val._snmp == '06' then -- OID
+		local oidStr = string.char(val[1]*40 + val[2])
+		for i = 3, #val do
+   			oidStr = oidStr .. self.encode_oid_component(val[i])
+		end 
+		return bin.pack("HAA", '06', self.encodeLength(#oidStr), oidStr) 
 
----
--- Encodes the length part of a ASN.1 encoding triplet using the "primitive,
--- definite-length" method.
--- @param val Value to be encoded.
--- @return Encoded length value.
-local function encodeLength(len)
-   if len < 128 then
-      return string.char(len)
-   else
-      local parts = {}
+ 	elseif (val._snmp == '40') then -- ipAddress
+		return bin.pack("HC4", '40 04', unpack(val))
+ 	
+    -- counter or gauge or timeticks or opaque
+	elseif (val._snmp == '41' or val._snmp == '42' or val._snmp == '43' or val._snmp == '44') then
+		local val = self:encodeInt(val[1])
+		return bin.pack("HAA", val._snmp, self.encodeLength(#val), val)
+	end
+	
+	local encVal = ""
+	for _, v in ipairs(val) do
+		encVal = encVal .. self:encode(v) -- todo: buffer?
+	end
 
-      while len > 0 do
-         parts[#parts + 1] = string.char(bit.mod(len, 256))
-         len = bit.rshift(len, 8)
-      end
-
-      assert(#parts < 128)
-      return string.char(#parts + 0x80) .. string.reverse(table.concat(parts))
-   end
+	local tableType = bin.pack("H", "30")
+	if (val["_snmp"]) then 
+		tableType = bin.pack("H", val["_snmp"]) 
+	end
+	return bin.pack('AAA', tableType, self.encodeLength(#encVal), encVal)
 end
 
-
--- Encode one component of an OID as a byte string. 7 bits of the component are
--- stored in each octet, most significant first, with the eigth bit set in all
--- octets but the last. These encoding rules come from
--- http://luca.ntop.org/Teaching/Appunti/asn1.html, section 5.9 OBJECT
--- IDENTIFIER.
-local function encode_oid_component(n)
-  local parts = {}
-  parts[1] = string.char(bit.mod(n, 128))
-  while n >= 128 do
-    n = bit.rshift(n, 7)
-    parts[#parts + 1] = string.char(bit.mod(n, 128) + 0x80)
-  end
-  return string.reverse(table.concat(parts))
-end
-
-
 ---
 -- Encodes a given value according to ASN.1 basic encoding rules for SNMP
 -- packet creation.
 -- @param val Value to be encoded.
 -- @return Encoded value.
 function encode(val)
-   local vtype = type(val)
-   if (vtype == 'number') then
-      local ival = encodeInt(val)
-      local len = encodeLength(string.len(ival))
-      return bin.pack('HAA', '02', len, ival)
-   end
-   if (vtype == 'string') then
-      local len = encodeLength(string.len(val))
-      return bin.pack('HAA', '04', len, val)
-   end
-   if (vtype == 'nil' or vtype == 'boolean') then
-      return bin.pack('H', '05 00')
-   end
-   if (vtype == 'table') then -- complex data types
-      if val._snmp == '06' then -- OID
-	 local oidStr = string.char(val[1]*40 + val[2])
-	 for i = 3, #val do
-	    oidStr = oidStr .. encode_oid_component(val[i])
-	 end 
-	 return bin.pack("HAA", '06', encodeLength(#oidStr), oidStr) 
-      elseif (val._snmp == '40') then -- ipAddress
-	 return bin.pack("HC4", '40 04', unpack(val))
-      elseif (val._snmp == '41') then -- counter
-	 local cnt = encodeInt(val[1])
-	 return bin.pack("HAA", val._snmp, encodeLength(string.len(cnt)), cnt)
-      elseif (val._snmp == '42') then -- gauge
-	 local gauge = encodeInt(val[1])
-	 return bin.pack("HAA", val._snmp, encodeLength(string.len(gauge)), gauge)
-      elseif (val._snmp == '43') then -- timeticks
-	 local ticks = encodeInt(val[1])
-	 return bin.pack("HAA", val._snmp, encodeLength(string.len(ticks)), ticks)
-      elseif (val._snmp == '44') then -- opaque
-	 return bin.pack("HAA", val._snmp, encodeLength(string.len(val[1])), val[1])
-      end
-      local encVal = ""
-      for _, v in ipairs(val) do
-	 encVal = encVal .. encode(v) -- todo: buffer?
-      end
-      local tableType = bin.pack("H", "30")
-      if (val["_snmp"]) then 
-	 tableType = bin.pack("H", val["_snmp"]) 
-      end
-      return bin.pack('AAA', tableType, encodeLength(string.len(encVal)), encVal)
-   end
-   return ''
+	local vtype = type(val)
+	local encoder = asn1.ASN1Encoder:new()
+	encoder:registerTagEncoders( tagEncoder )
+   	
+
+	local encVal = encoder:encode(val)
+		
+	if encVal then
+		return encVal
+	end
+	
+	return ''
 end
 
+-- SNMP ASN.1 Decoders
+local tagDecoder = {}
 
----
--- Decodes length part of encoded value according to ASN.1 basic encoding
--- rules.
--- @param encStr Encoded string.
--- @param pos Current position in the string.
--- @return The position after decoding.
--- @return The length of the following value.
-local function decodeLength(encStr, pos)
-   local elen
-   pos, elen = bin.unpack('C', encStr, pos)
-   if (elen > 128) then
-      elen = elen - 128
-      local elenCalc = 0
-      local elenNext
-      for i = 1, elen do
-	 elenCalc = elenCalc * 256
-	 pos, elenNext = bin.unpack("C", encStr, pos)
-	 elenCalc = elenCalc + elenNext
-      end
-      elen = elenCalc
-   end
-   return pos, elen
-end
-
-
----
--- Decodes an Integer according to ASN.1 basic encoding rules.
--- @param encStr Encoded string.
--- @param len Length of integer in bytes.
--- @param pos Current position in the string.
--- @return The position after decoding.
--- @return The decoded integer.
-local function decodeInt(encStr, len, pos)
-   local hexStr
-   pos, hexStr = bin.unpack("H" .. len, encStr, pos)
-   local value = tonumber(hexStr, 16)
-   if (value >= math.pow(256, len)/2) then
-      value = value - math.pow(256, len)
-   end
-   return pos, value
-end
-
--- Decode one component of an OID from a byte string. 7 bits of the component
--- are stored in each octet, most significant first, with the eigth bit set in
--- all octets but the last. These encoding rules come from
--- http://luca.ntop.org/Teaching/Appunti/asn1.html, section 5.9 OBJECT
--- IDENTIFIER.
-local function decode_oid_component(encStr, pos)
-   local octet
-   local n = 0
-
-   repeat
-      pos, octet = bin.unpack("C", encStr, pos)
-      n = n * 128 + bit.band(0x7F, octet)
-   until octet < 128
-
-   return pos, n
-end
-
---- Decodes an OID from a sequence of bytes.
+-- Application specific tags
 --
--- @param encStr Encoded string.
--- @param len Length of sequence in bytes.
--- @param pos Current position in the string.
--- @return The position after decoding.
--- @return The OID as an array.
-local function decodeOID(encStr, len, pos)
-   local last
-   local oid = {}
-   local octet
+-- IP Address
 
-   last = pos + len - 1
-   if pos <= last then
-      oid._snmp = '06'
-      pos, octet = bin.unpack("C", encStr, pos)
-      oid[2] = math.mod(octet, 40)
-      octet = octet - oid[2]
-      oid[1] = octet/40
-   end
-
-   while pos <= last do
-      local c
-      pos, c = decode_oid_component(encStr, pos)
-      oid[#oid + 1] = c
-   end
+-- Response-PDU
+-- TOOD: Figure out how to remove these dependancies
+tagDecoder["A2"] = function( self, encStr, elen, pos )
+   local seq = {}
  
-   return pos, oid
+   pos, seq = self:decodeSeq(encStr, elen, pos)
+   seq._snmp = "A2"
+   return pos, seq
 end
 
----
--- Decodes a sequence according to ASN.1 basic encoding rules.
--- @param encStr Encoded string.
--- @param len Length of sequence in bytes.
--- @param pos Current position in the string.
--- @return The position after decoding.
--- @return The decoded sequence as a table.
-local function decodeSeq(encStr, len, pos)
-   local seq = {}
-   local sPos = 1
-   local i = 1
-   local sStr
-   pos, sStr = bin.unpack("A" .. len, encStr, pos)
-   while (sPos < len) do
-      local newSeq
-      sPos, newSeq = decode(sStr, sPos)
-      table.insert(seq, newSeq)
-      i = i + 1
-   end
-   return pos, seq
+tagDecoder["40"] = function( self, encStr, elen, pos )
+	local ip = {}
+	pos, ip[1], ip[2], ip[3], ip[4] = bin.unpack("C4", encStr, pos)
+	ip._snmp = '40'
+	return pos, ip
 end
 
 ---
@@ -259,81 +101,35 @@ end
 -- @return The position after decoding
 -- @return The decoded value(s).
 function decode(encStr, pos)
-   local etype, elen
-   pos, etype = bin.unpack("H1", encStr, pos)
-   pos, elen = decodeLength(encStr, pos)
-   if (etype == "02") then -- INTEGER
-      return decodeInt(encStr, elen, pos)
-      
-   elseif (etype == "04") then -- STRING
-      return bin.unpack("A" .. elen, encStr, pos)
-      
-   elseif (etype == "05") then -- NULL
-      return pos, false
+	local decoder = asn1.ASN1Decoder:new()
+	
+	if ( #tagDecoder == 0 ) then
+		decoder:registerBaseDecoders()
+		-- Application specific tags
+		-- tagDecoder["40"] = decoder.decoder["06"]  -- IP Address; same as OID
+		tagDecoder["41"] = decoder.decoder["02"]  -- Counter; same as Integer
+		tagDecoder["42"] = decoder.decoder["02"]  -- Gauge
+		tagDecoder["43"] = decoder.decoder["02"]  -- TimeTicks
+		tagDecoder["44"] = decoder.decoder["04"]  -- Opaque; same as Octet String
+		tagDecoder["45"] = decoder.decoder["06"]  -- NsapAddress
+		tagDecoder["46"] = decoder.decoder["02"]  -- Counter64
+		tagDecoder["47"] = decoder.decoder["02"]  -- UInteger32
 
-   elseif (etype == "06") then -- OID
-      return decodeOID( encStr, elen, pos )
+		-- Context specific tags
+		tagDecoder["A0"] = decoder.decoder["30"]  -- GetRequest-PDU
+		tagDecoder["A1"] = decoder.decoder["30"]  -- GetNextRequest-PDU
+		--tagDecoder["A2"] = decoder.decoder["30"]  -- Response-PDU
+		tagDecoder["A3"] = decoder.decoder["30"]  -- SetRequest-PDU
+		tagDecoder["A4"] = decoder.decoder["30"]  -- Trap-PDU
+		tagDecoder["A5"] = decoder.decoder["30"]  -- GetBulkRequest-PDU
+		tagDecoder["A6"] = decoder.decoder["30"]  -- InformRequest-PDU (not implemented here yet)
+		tagDecoder["A7"] = decoder.decoder["30"]  -- SNMPv2-Trap-PDU (not implemented here yet)
+	end
+	
+	
+	decoder:registerTagDecoders( tagDecoder )
 
-   elseif (etype == "30") then -- sequence
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      return pos, seq
-
-   elseif (etype == "A0") then -- getReq
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      seq._snmp = etype
-      return pos, seq
-
-   elseif (etype == "A1") then -- getNextReq
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      seq._snmp = etype
-      return pos, seq
-
-   elseif (etype == "A2") then -- getResponse
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      seq._snmp = etype
-      return pos, seq
-
-   elseif (etype == "A3") then -- setReq
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      seq._snmp = etype
-      return pos, seq
-   elseif (etype == "A4") then -- Trap
-      local seq
-      pos, seq = decodeSeq(encStr, elen, pos)
-      seq._snmp = etype
-      return pos, seq
-   elseif (etype == '40') then -- App: IP-Address
-      local ip = {}
-      pos, ip[1], ip[2], ip[3], ip[4] = bin.unpack("C4", encStr, pos)
-      ip._snmp = '40'
-      return pos, ip
-   elseif (etype == '41') then -- App: counter
-      local cnt = {}
-      pos, cnt[1] = decodeInt(encStr, elen, pos)
-      cnt._snmp = '41'
-      return pos, cnt
-   elseif (etype == '42') then -- App: gauge
-      local gauge = {}
-      pos, gauge[1] = decodeInt(encStr, elen, pos)
-      gauge._snmp = '42'
-      return pos, gauge
-   elseif (etype == '43') then -- App: TimeTicks
-      local ticks = {}
-      pos, ticks[1] = decodeInt(encStr, elen, pos)
-      ticks._snmp = '43'
-      return pos, ticks
-   elseif (etype == '44') then -- App: opaque
-      local opaque = {}
-      pos, opaque[1] = bin.unpack("A" .. elen, encStr, pos)
-      opaque._snmp = '44'
-      return pos, opaque
-   end
-   return pos, nil
+   return decoder:decode( encStr, pos )
 end
 
 ---
@@ -416,7 +212,7 @@ end
 function buildGetNextRequest(options, ...)
    if not options then options = {} end
 
-	 if not options.reqId then options.reqId = math.mod(nmap.clock_ms(), 65000) end
+   if not options.reqId then options.reqId = math.mod(nmap.clock_ms(), 65000) end
    if not options.err then options.err = 0 end
    if not options.errIdx then options.errIdx = 0 end
 
@@ -431,7 +227,7 @@ function buildGetNextRequest(options, ...)
       payload[i] = {}
       payload[i][1] = select(i, ...)
       if type(payload[i][1]) == "string" then
-	 payload[i][1] = str2oid(payload[i][1])
+	     payload[i][1] = str2oid(payload[i][1])
       end
       payload[i][2] = false
    end
@@ -648,8 +444,68 @@ end
 -- @param response SNMP Response (will be decoded if necessary).
 -- @return First decoded value of the response.
 function fetchFirst(response)
-   local result = fetchResponseValues(response)
-   if type(result) == "table" and result[1] and result[1][1] then return result[1][1]
-   else return nil
-   end
+	local result = fetchResponseValues(response)
+
+	if type(result) == "table" and result[1] and result[1][1] then
+		return result[1][1]
+   	else 
+		return nil
+	end
+end
+
+
+--- Walks the MIB Tree
+--
+-- @param socket socket already connected to the server
+-- @param base_oid string containing the base object ID to walk
+-- @return status true on success, false on failure
+-- @return table containing <code>oid</code> and <code>value</code>
+function snmpWalk( socket, base_oid )
+	
+	local snmp_table = {}
+	local oid = base_oid
+	local status, err, payload
+	
+	while ( true ) do
+		
+		local value, response, snmpdata, options, item = nil, nil, nil, {}, {}
+		options.reqId = 28428 -- unnecessary?
+		payload = snmp.encode( snmp.buildPacket( snmp.buildGetNextRequest(options, oid) ) )
+
+		status, err = socket:send(payload)
+		if ( not( status ) ) then
+			stdnse.print_debug("snmp.snmpWalk: Send failed")
+			return false, err
+		end
+		
+		status, response = socket:receive_bytes(1) 
+		if ( not( status ) ) then
+			-- Unless we don't have a usefull error message, don't report it
+			if ( response ~= "ERROR" ) then
+				stdnse.print_debug("snmp.snmpWalk: Received no answer (%s)", response)
+				return false, response
+			end
+			return false, nil
+		end
+	
+		snmpdata = snmp.fetchResponseValues( response )
+		
+		value = snmpdata[1][1]
+		oid  = snmpdata[1][2]
+		
+		if not oid:match( base_oid ) or base_oid == oid then
+			break
+		end
+		
+		item.oid = oid
+		item.value = value
+		
+		table.insert( snmp_table, item )
+		
+	end
+
+	snmp_table.baseoid = base_oid
+
+	return true, snmp_table
+	
 end
