@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -16,52 +16,45 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -77,35 +70,36 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
+ * distribution.  By sending these changes to Fyodor or one of the         *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.h 6633 2007-12-22 06:32:03Z fyodor $ */
+/* $Id: tcpip.h 28208 2012-03-04 07:26:16Z batrick $ */
 
 
 #ifndef TCPIP_H
 #define TCPIP_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "nmap_config.h"
 #endif
 
 #include "nbase.h"
@@ -142,19 +136,6 @@ void *realloc();
 #include <sys/param.h> /* Defines MAXHOSTNAMELEN on BSD*/
 #endif
 
-/* Linux uses these defines in netinet/ip.h to use the correct struct ip */
-#ifndef __FAVOR_BSD
-#define __FAVOR_BSD 1
-#endif
-#ifndef _BSD_SOURCE
-#define _BSD_SOURCE 1
-#endif
-#ifndef __USE_BSD
-#define __USE_BSD 1
-#endif
-/* BSDI needs this to insure the correct struct ip */
-#undef _IP_VHL
-
 #include <stdio.h>
 
 #if HAVE_NETINET_IN_H
@@ -174,14 +155,6 @@ void *realloc();
 #endif
 
 #include <arpa/inet.h>
-#ifndef NETINET_IN_SYSTEM_H  /* why the HELL does OpenBSD not do this? */
-#include <netinet/in_systm.h> /* defines n_long needed for netinet/ip.h */
-#define NETINET_IN_SYSTEM_H
-#endif
-#ifndef NETINET_IP_H  /* why the HELL does OpenBSD not do this? */
-#include <netinet/ip.h>
-#define NETINET_IP_H
-#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -217,11 +190,8 @@ extern "C" {
 #include <errno.h>
 #include <signal.h>
 #include <dnet.h>
-#ifndef WIN32
-#include <netinet/ip_icmp.h>
-#endif
 
-typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;
+
 
 #include "nmap.h"
 #include "global_structures.h"
@@ -232,12 +202,13 @@ typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;
 
 
 /* Used for tracing all packets sent or received (eg the
-   --packet_trace option) */
+   --packet-trace option) */
+
 class PacketTrace {
  public:
-  /*  static const int SEND=1;
-      static const int RCV=2; */
-  enum pdirection { SENT=1, RCVD=2 };
+  static const int SENT=1; /* These two values must not be changed */
+  static const int RCVD=2;
+  typedef int pdirection;
   /* Takes an IP PACKET and prints it if packet tracing is enabled.
      'packet' must point to the IPv4 header. The direction must be
      PacketTrace::SENT or PacketTrace::RCVD .  Optional 'now' argument
@@ -261,6 +232,8 @@ class PacketTrace {
      efficient by avoiding a gettimeofday() call. */
   static void traceArp(pdirection pdir, const u8 *frame, u32 len,
 				    struct timeval *now);
+  static void traceND(pdirection pdir, const u8 *frame, u32 len,
+                                    struct timeval *now);
 };
 
 class PacketCounter {
@@ -274,164 +247,6 @@ class PacketCounter {
 	  sendPackets, sendBytes, recvPackets, recvBytes;
 };
 
-#define MAX_LINK_HEADERSZ 24
-struct link_header {
-  int datalinktype; /* pcap_datalink(), such as DLT_EN10MB */
-  int headerlen; /* 0 if header was too big or unavailaable */
-  u8 header[MAX_LINK_HEADERSZ];
-};
-
-/* Relevant (to Nmap) information about an interface */
-struct interface_info {
-  char devname[16];
-  char devfullname[16]; /* can include alias info, such as eth0:2. */
-  struct sockaddr_storage addr;
-  u16 netmask_bits; /* CIDR-style.  So 24 means class C (255.255.255.0)*/
-  devtype device_type; /* devt_ethernet, devt_loopback, devt_p2p, devt_other */
-  bool device_up; /* True if the device is up (enabled) */
-  u8 mac[6]; /* Interface MAC address if device_type is devt_ethernet */
-};
-
-struct route_nfo {
-  struct interface_info ii;
-
-/* true if the target is directly connected on the network (no routing
-   required). */
-  bool direct_connect; 
-
-/* This is the source address that should be used by the packets.  It
-   may be different than ii.addr if you are using localhost interface
-   to scan the IP of another interface on the machine */
-  struct sockaddr_storage srcaddr; 
-
-  /* If direct_connect is 0, this is filled in with the next hop
-     required to route to the target */
-  struct sockaddr_storage nexthop;
-};
-
-struct sys_route {
-  struct interface_info *device;
-  u32 dest;
-  u32 netmask;
-  struct in_addr gw; /* gateway - 0 if none */
-};
-
-struct eth_nfo {
-  char srcmac[6];
-  char dstmac[6];
-  eth_t *ethsd; // Optional, but improves performance.  Set to NULL if unavail
-  char devname[16]; // Only needed if ethsd is NULL.
-};
-
-#ifndef HAVE_STRUCT_IP
-#define HAVE_STRUCT_IP
-
-/* From Linux glibc, which apparently borrowed it from
-   BSD code.  Slightly modified for portability --fyodor@insecure.org */
-/*
- * Structure of an internet header, naked of options.
- */
-struct ip
-  {
-#if WORDS_BIGENDIAN
-    u_int8_t ip_v:4;                    /* version */
-    u_int8_t ip_hl:4;                   /* header length */
-#else
-    u_int8_t ip_hl:4;                   /* header length */
-    u_int8_t ip_v:4;                    /* version */ 
-#endif
-    u_int8_t ip_tos;                    /* type of service */
-    u_short ip_len;                     /* total length */
-    u_short ip_id;                      /* identification */
-    u_short ip_off;                     /* fragment offset field */
-#define IP_RF 0x8000                    /* reserved fragment flag */
-#define IP_DF 0x4000                    /* dont fragment flag */
-#define IP_MF 0x2000                    /* more fragments flag */
-#define IP_OFFMASK 0x1fff               /* mask for fragmenting bits */
-    u_int8_t ip_ttl;                    /* time to live */
-    u_int8_t ip_p;                      /* protocol */
-    u_short ip_sum;                     /* checksum */
-    struct in_addr ip_src, ip_dst;      /* source and dest address */
-  };
-
-#endif /* HAVE_STRUCT_IP */
-
-#ifndef HAVE_STRUCT_ICMP
-#define HAVE_STRUCT_ICMP
-/* From Linux /usr/include/netinet/ip_icmp.h GLIBC */
-
-/*
- * Internal of an ICMP Router Advertisement
- */
-struct icmp_ra_addr
-{
-  u_int32_t ira_addr;
-  u_int32_t ira_preference;
-};
-
-struct icmp
-{
-  u_int8_t  icmp_type;  /* type of message, see below */
-  u_int8_t  icmp_code;  /* type sub code */
-  u_int16_t icmp_cksum; /* ones complement checksum of struct */
-  union
-  {
-    struct ih_idseq             /* echo datagram */
-    {
-      u_int16_t icd_id;
-      u_int16_t icd_seq;
-    } ih_idseq;
-    u_int32_t ih_void;
-
-    /* ICMP_UNREACH_NEEDFRAG -- Path MTU Discovery (RFC1191) */
-    struct ih_pmtu
-    {
-      u_int16_t ipm_void;
-      u_int16_t ipm_nextmtu;
-    } ih_pmtu;
-
-    struct ih_rtradv
-    {
-      u_int8_t irt_num_addrs;
-      u_int8_t irt_wpa;
-      u_int16_t irt_lifetime;
-    } ih_rtradv;
-  } icmp_hun;
-  /* Removed icmp_pptr and icmp_gwaddr from union and #defines because they conflict with dnet */
-#define icmp_id         icmp_hun.ih_idseq.icd_id
-#define icmp_seq        icmp_hun.ih_idseq.icd_seq
-#define icmp_void       icmp_hun.ih_void
-#define icmp_pmvoid     icmp_hun.ih_pmtu.ipm_void
-#define icmp_nextmtu    icmp_hun.ih_pmtu.ipm_nextmtu
-#define icmp_num_addrs  icmp_hun.ih_rtradv.irt_num_addrs
-#define icmp_wpa        icmp_hun.ih_rtradv.irt_wpa
-#define icmp_lifetime   icmp_hun.ih_rtradv.irt_lifetime
-  union
-  {
-    struct
-    {
-      u_int32_t its_otime;
-      u_int32_t its_rtime;
-      u_int32_t its_ttime;
-    } id_ts;
-    struct
-    {
-      struct ip idi_ip;
-      /* options and then 64 bits of data */
-    } id_ip;
-    struct icmp_ra_addr id_radv;
-    u_int32_t   id_mask;
-    u_int8_t    id_data[1];
-  } icmp_dun;
-#define icmp_otime      icmp_dun.id_ts.its_otime
-#define icmp_rtime      icmp_dun.id_ts.its_rtime
-#define icmp_ttime      icmp_dun.id_ts.its_ttime
-#define icmp_ip         icmp_dun.id_ip.idi_ip
-#define icmp_radv       icmp_dun.id_radv
-#define icmp_mask       icmp_dun.id_mask
-#define icmp_data       icmp_dun.id_data
-};
-#endif /* HAVE_STRUCT_ICMP */
 
 /* Some systems might not have this */
 #ifndef IPPROTO_IGMP
@@ -444,18 +259,12 @@ struct icmp
    not thread-safe and can only be used once in calls like printf() 
 */
 const char *inet_socktop(struct sockaddr_storage *ss);
+
 /* Tries to resolve the given name (or literal IP) into a sockaddr
-   structure.  The af should be PF_INET (for IPv4) or PF_INET6.  Returns 0
-   if hostname cannot be resolved.  It is OK to pass in a sockaddr_in or 
-   sockaddr_in6 casted to a sockaddr_storage as long as you use the matching 
-   pf.*/
-int resolve(char *hostname, struct sockaddr_storage *ss, size_t *sslen,
-	    int pf);
-/* LEGACY resolve() function that only supports IPv4 -- see IPv6 version
-   above.  Tries to resolve given hostname and stores
-   result in ip .  returns 0 if hostname cannot
-   be resolved */
-int resolve(char *hostname, struct in_addr *ip);
+   structure. This function calls getaddrinfo and returns the same
+   addrinfo linked list that getaddrinfo produces. Returns NULL for any
+   error or failure to resolve. */
+struct addrinfo *resolve_all(const char *hostname, int pf);
 
 /* Takes a destination address (dst) and tries to determine the
    source address and interface necessary to route to this address.
@@ -463,48 +272,33 @@ int resolve(char *hostname, struct in_addr *ip);
    a route is found, true is returned and rnfo is filled in with all
    of the routing details.  This function takes into account -S and -e
    options set by user (o.spoofsource, o.device) */
-bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo);
-
-/* Determines what interface packets destined to 'dest' should be
-   routed through.  It can also discover the appropriate next hop (if
-   any) for ethernet routing.  If direct_connect is passed in, it will
-   be set to 1 if dst is directly connected on the ifentry network and
-   0 if it requires routing.  If nexthop_ip is not NULL, and routing
-   is required, the next hop is filled into nexthop_ip.  This function
-   returns false if no appropiate interface or route was found and
-   true if it succeeds. */
-bool routethrough(const struct sockaddr_storage * const dest, 
-		  struct intf_entry *ifentry, 
-		  int *direct_connect, struct sockaddr_storage *nexthop_ip);
+int nmap_route_dst(const struct sockaddr_storage *dst, struct route_nfo *rnfo);
 
 unsigned short in_cksum(u16 *ptr,int nbytes);
 
-unsigned short magic_tcpudp_cksum(const struct in_addr *src,
-				  const struct in_addr *dst,
-				  u8 proto, u16 len, char *hstart);
 
-/* Build and send a raw tcp packet.  If TTL is -1, a partially random
-   (but likely large enough) one is chosen */
-int send_tcp_raw( int sd, struct eth_nfo *eth,
-		  const struct in_addr *source, const struct in_addr *victim, 
-		  int ttl, bool df,
-		  u8* ipopt, int ipoptlen,
-		  u16 sport, u16 dport,
-		  u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
-		  u8 *options, int optlen,
-		  char *data, u16 datalen);
-int send_udp_raw( int sd, struct eth_nfo *eth,
-		  struct in_addr *source, const struct in_addr *victim,
-		  int ttl, u16 ipid,
-		  u8* ipopt, int ipoptlen,
-		  u16 sport, u16 dport,
-		  char *data, u16 datalen);
+/* Send a pre-built IPv4 or IPv6 packet */
+int send_ip_packet(int sd, const struct eth_nfo *eth,
+  const struct sockaddr_storage *dst,
+  const u8 *packet, unsigned int packetlen);
 
-int send_ip_raw( int sd, struct eth_nfo *eth,
-		 struct in_addr *source, const struct in_addr *victim,
-		 u8 proto, int ttl,
-		 u8* ipopt, int ipoptlen,
-		 char *data, u16 datalen);
+/* Builds an IP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_ip_raw(const struct in_addr *source, const struct in_addr *victim, 
+		 u8 proto,
+		 int ttl, u16 ipid, u8 tos, bool df,
+		 const u8* ipopt, int ipoptlen,
+		 const char *data, u16 datalen,
+		 u32 *packetlen);
+
+u8 *build_ipv6_raw(const struct in6_addr *source,
+                   const struct in6_addr *victim, u8 tc, u32 flowlabel,
+                   u8 nextheader, int hoplimit,
+                   char *data, u16 datalen, u32 *outpacketlen);
 
 /* Builds a TCP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
@@ -514,12 +308,39 @@ int send_ip_raw( int sd, struct eth_nfo *eth,
    packetlen, which must be a valid int pointer. */
 u8 *build_tcp_raw(const struct in_addr *source, const struct in_addr *victim,
 		  int ttl, u16 ipid, u8 tos, bool df,
+		  const u8* ipopt, int ipoptlen,
+		  u16 sport, u16 dport,
+		  u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
+		  const u8 *options, int optlen,
+		  char *data, u16 datalen,
+		  u32 *packetlen);
+
+u8 *build_tcp_raw_ipv6(const struct in6_addr *source,
+                       const struct in6_addr *victim, u8 tc, u32 flowlabel,
+                       u8 hoplimit, u16 sport, u16 dport, u32 seq, u32 ack,
+                       u8 reserved, u8 flags, u16 window, u16 urp,
+                       const u8 *tcpopt, int tcpoptlen, char *data,
+                       u16 datalen, u32 *packetlen);
+
+/* Build and send a raw tcp packet.  If TTL is -1, a partially random
+   (but likely large enough) one is chosen */
+int send_tcp_raw( int sd, const struct eth_nfo *eth,
+		  const struct in_addr *source, const struct in_addr *victim, 
+		  int ttl, bool df,
 		  u8* ipopt, int ipoptlen,
 		  u16 sport, u16 dport,
 		  u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
 		  u8 *options, int optlen,
-		  char *data, u16 datalen,
-		  u32 *packetlen);
+		  char *data, u16 datalen);
+
+int send_tcp_raw_decoys( int sd, const struct eth_nfo *eth, 
+			 const struct in_addr *victim,
+			 int ttl, bool df, 
+			 u8* ipopt, int ipoptlen,
+			 u16 sport, u16 dport,
+			 u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
+			 u8 *options, int optlen,
+			 char *data, u16 datalen);
 
 /* Builds a UDP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
@@ -527,13 +348,52 @@ u8 *build_tcp_raw(const struct in_addr *source, const struct in_addr *victim,
    actually sent by this function.  Caller must delete the buffer when
    finished with the packet.  The packet length is returned in
    packetlen, which must be a valid int pointer. */
-u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
+u8 *build_udp_raw(const struct in_addr *source, const struct in_addr *victim,
  		  int ttl, u16 ipid, u8 tos, bool df,
 		  u8* ipopt, int ipoptlen,
  		  u16 sport, u16 dport, 
- 		  char *data, u16 datalen,
+ 		  const char *data, u16 datalen,
  		  u32 *packetlen);
 
+u8 *build_udp_raw_ipv6(const struct in6_addr *source,
+                       const struct in6_addr *victim, u8 tc, u32 flowlabel,
+                       u8 hoplimit, u16 sport, u16 dport,
+                       char *data, u16 datalen, u32 *packetlen);
+
+int send_udp_raw( int sd, const struct eth_nfo *eth,
+		  struct in_addr *source, const struct in_addr *victim,
+		  int ttl, u16 ipid,
+		  u8* ipopt, int ipoptlen,
+		  u16 sport, u16 dport,
+		  char *data, u16 datalen);
+
+int send_udp_raw_decoys( int sd, const struct eth_nfo *eth, 
+			 const struct in_addr *victim,
+			 int ttl, u16 ipid,
+			 u8* ipops, int ip,
+			 u16 sport, u16 dport,
+			 char *data, u16 datalen);
+
+/* Builds an SCTP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_sctp_raw(const struct in_addr *source, const struct in_addr *victim,
+		   int ttl, u16 ipid, u8 tos, bool df,
+		   u8* ipopt, int ipoptlen,
+		   u16 sport, u16 dport,
+		   u32 vtag, char *chunks, int chunkslen,
+		   char *data, u16 datalen,
+		   u32 *packetlen);
+
+u8 *build_sctp_raw_ipv6(const struct in6_addr *source,
+                        const struct in6_addr *victim, u8 tc, u32 flowlabel,
+                        u8 hoplimit, u16 sport, u16 dport, u32 vtag,
+                        char *chunks, int chunkslen, char *data, u16 datalen,
+                        u32 *packetlen);
+
 /* Builds an ICMP packet (including an IP header) by packing the
    fields with the given information.  It allocates a new buffer to
    store the packet contents, and then returns that buffer.  The
@@ -548,6 +408,11 @@ u8 *build_icmp_raw(const struct in_addr *source, const struct in_addr *victim,
 		   u16 seq, unsigned short id, u8 ptype, u8 pcode,
 		   char *data, u16 datalen, u32 *packetlen);
 
+u8 *build_icmpv6_raw(const struct in6_addr *source,
+                     const struct in6_addr *victim, u8 tc, u32 flowlabel,
+                     u8 hoplimit, u16 seq, u16 id, u8 ptype, u8 pcode,
+                     char *data, u16 datalen, u32 *packetlen);
+
 /* Builds an IGMP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
    packet contents, and then returns that buffer.  The packet is not
@@ -561,56 +426,6 @@ u8 *build_igmp_raw(const struct in_addr *source, const struct in_addr *victim,
 		   u8 ptype, u8 pcode,
 		   char *data, u16 datalen, u32 *packetlen);
 
-/* Builds an IP packet (including an IP header) by packing the fields
-   with the given information.  It allocates a new buffer to store the
-   packet contents, and then returns that buffer.  The packet is not
-   actually sent by this function.  Caller must delete the buffer when
-   finished with the packet.  The packet length is returned in
-   packetlen, which must be a valid int pointer. */
-u8 *build_ip_raw(const struct in_addr *source, const struct in_addr *victim, 
-		 u8 proto,
-		 int ttl, u16 ipid, u8 tos, bool df,
-		 u8* ipopt, int ipoptlen,
-		 char *data, u16 datalen, 
-		 u32 *packetlen);
-
-/* Send a pre-built IPv4 packet */
-int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, 
-		   unsigned int packetlen);
-
-/* Decoy versions of the raw packet sending functions ... */
-int send_tcp_raw_decoys( int sd, struct eth_nfo *eth, 
-			 const struct in_addr *victim,
-			 int ttl, bool df, 
-			 u8* ipopt, int ipoptlen,
-			 u16 sport, u16 dport,
-			 u32 seq, u32 ack, u8 reserved, u8 flags, u16 window, u16 urp,
-			 u8 *options, int optlen,
-			 char *data, u16 datalen);
-
-int send_udp_raw_decoys( int sd, struct eth_nfo *eth, 
-			 const struct in_addr *victim,
-			 int ttl, u16 ipid,
-			 u8* ipops, int ip,
-			 u16 sport, u16 dport,
-			 char *data, u16 datalen);
-
-
-/* Calls pcap_open_live and spits out an error (and quits) if the call fails.
-   So a valid pcap_t will always be returned. */
-pcap_t *my_pcap_open_live(const char *device, int snaplen, int promisc, 
-			  int to_ms);
-
-// Returns whether the system supports pcap_get_selectable_fd() properly
-bool pcap_selectable_fd_valid();
-
-/* Call this instead of pcap_get_selectable_fd directly (or your code
-   won't compile on Windows).  On systems which don't seem to support
-   the pcap_get_selectable_fd() function properly, returns -1,
-   otherwise simply calls pcap_selectable_fd and returns the
-   results.  If you just want to test whether the function is supported,
-   use pcap_selectable_fd_valid() instead. */
-int my_pcap_get_selectable_fd(pcap_t *p);
 
 // Returns whether the packet receive time value obtaned from libpcap
 // (and thus by readip_pcap()) should be considered valid.  When
@@ -621,49 +436,20 @@ bool pcap_recv_timeval_valid();
    packets). */
 void pcap_print_stats(int logt, pcap_t *pd);
 
-/* A simple function that caches the eth_t from dnet for one device,
-   to avoid opening, closing, and re-opening it thousands of tims.  If
-   you give a different device, this function will close the first
-   one.  Thus this should never be used by programs that need to deal
-   with multiple devices at once.  In addition, you MUST NEVER
-   eth_close() A DEVICE OBTAINED FROM THIS FUNCTION.  Instead, you can
-   call eth_close_cached() to close whichever device (if any) is
-   cached.  Returns NULL if it fails to open the device. */
-eth_t *eth_open_cached(const char *device);
 
-/* See the description for eth_open_cached */
-void eth_close_cached();
 
 /* A simple function I wrote to help in debugging, shows the important fields
    of a TCP packet*/
 int readtcppacket(const u8 *packet, int readdata);
 int readudppacket(const u8 *packet, int readdata);
-/* Convert an IP address to the device (IE ppp0 eth0) using that address.  Dev passed in must be at least 
-    32 bytes long */
-int ipaddr2devname( char *dev, const struct in_addr *addr );
-/* And vice versa */
-int devname2ipaddr(char *dev, struct in_addr *addr);
+
 /* Looks for an interface assigned to the given IP (ss), and returns
    the interface_info for the first one found.  If non found, returns NULL */
 struct interface_info *getInterfaceByIP(struct sockaddr_storage *ss);
-/* Looks for an interface with the given name (iname), and returns the
-   corresponding interface_info if found.  Will accept a match of
-   devname or devfullname.  Returns NULL if none found */
-struct interface_info *getInterfaceByName(char *iname);
-/* Where the above 4 functions get their info */
-struct interface_info *getinterfaces(int *howmany);
+
+
 pcap_if_t *getpcapinterfaces();
 
-/* Parse the system routing table, converting each route into a
-   sys_route entry.  Returns an array of sys_routes.  numroutes is set
-   to the number of routes in the array.  The routing table is only
-   read the first time this is called -- later results are cached.
-   The returned route array is sorted by netmask with the most
-   specific matches first. */
-struct sys_route *getsysroutes(int *howmany);
-void sethdrinclude(int sd);
-
-void set_ttl(int sd, int ttl);
 
 /* Fill buf (up to buflen -- truncate if necessary but always
    terminate) with a short representation of the packet stats.
@@ -679,13 +465,10 @@ char *getFinalPacketStats(char *buf, int buflen);
       directly connected to the src host running Nmap.  If it is, set the MAC.
 
    This function returns 0 if it ends up setting the MAC, nonzero otherwise
-
-   This function assumes that ip has already been verified as
-   containing a complete IP header (or at least the first 20 bytes).
 */  
 
 int setTargetMACIfAvailable(Target *target, struct link_header *linkhdr,
-			    struct ip *ip, int overwrite);
+                            const struct sockaddr_storage *src, int overwrite);
 
 /* This function ensures that the next hop MAC address for a target is
    filled in.  This address is the target's own MAC if it is directly
@@ -696,24 +479,25 @@ int setTargetMACIfAvailable(Target *target, struct link_header *linkhdr,
    after an ARP scan if many directly connected machines are involved. */
 bool setTargetNextHopMAC(Target *target);
 
-int islocalhost(const struct in_addr * const addr);
-int isipprivate(const struct in_addr * const addr);
-int unblock_socket(int sd);
+bool getNextHopMAC(const char *iface, const u8 *srcmac, const struct sockaddr_storage *srcss,
+		   const struct sockaddr_storage *dstss, u8 *dstmac);
+
+
 
-// Takes a protocol number like IPPROTO_TCP, IPPROTO_UDP, or
-// IPPROTO_IP and returns a ascii representation (or "unknown" if it
-// doesn't recognize the number).  If uppercase is true, the returned
-// value will be in all uppercase letters.  You can skip this
-// parameter to use lowercase.
-const char *proto2ascii(u8 proto, bool uppercase=false);
 /* Hex dump */
 int get_link_offset(char *device);
 /* If rcvdtime is non-null and a packet is returned, rcvd will be
    filled with the time that packet was captured from the wire by
    pcap.  If linknfo is not NULL, lnknfo->headerlen and
    lnkinfo->header will be filled with the appropriate values. */
-char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec, 
-		  struct timeval *rcvdtime, struct link_header *linknfo);
+char *readipv4_pcap(pcap_t *pd, unsigned int *len, long to_usec, 
+                    struct timeval *rcvdtime, struct link_header *linknfo, bool validate);
+
+char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec,
+                  struct timeval *rcvdtime, struct link_header *linknfo, bool validate);
+
+int read_na_pcap(pcap_t *pd, u8 *sendermac, struct sockaddr_in6 *senderIP, long to_usec,
+                  struct timeval *rcvdtime, bool *has_mac);
 
 /* Attempts to read one IPv4/Ethernet ARP reply packet from the pcap
    descriptor pd.  If it receives one, fills in sendermac (must pass
@@ -740,13 +524,6 @@ int gettcpopt_ts(struct tcp_hdr *tcp, u32 *timestamp, u32 *echots);
 /* Maximize the receive buffer of a socket descriptor (up to 500K) */
 void max_rcvbuf(int sd);
 
-/* Maximize the open file descriptor limit for this process go up to the
-   max allowed  */
-int max_sd();
-
-/* Convert a socket to blocking mode */
-int block_socket(int sd);
-
 /* Give broadcast permission to a socket */
 void broadcast_socket(int sd);
 
@@ -760,7 +537,7 @@ int recvtime(int sd, char *buf, int len, int seconds, int *timedout);
 
 /* Sets a pcap filter function -- makes SOCK_RAW reads easier */
 #ifndef WINIP_H
-void set_pcap_filter(const char *device, pcap_t *pd, char *bpf, ...);
+void set_pcap_filter(const char *device, pcap_t *pd, const char *bpf, ...);
 #endif
 
 #endif /*TCPIP_H*/
