@@ -3,7 +3,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -13,11 +13,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -31,25 +32,26 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -67,15 +69,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -106,7 +109,7 @@ necessary to send two probes per target: one at the distance of the target to
 get a response, and one at distance - 1 to get a cache hit. When the distance
 isn't known in advance, the algorithm arbitrarily starts at a TTL of 10 and
 counts downward, then counts upward from 11 until it reaches the target. So a
-typeical trace may look like
+typical trace may look like
 
 TTL 10 -> TTL_EXCEEDED
 TTL  9 -> TTL_EXCEEDED
@@ -116,7 +119,7 @@ TTL 11 -> TTL_EXCEEDED
 TTL 12 -> TTL_EXCEEDED
 TTL 13 -> SYN/ACK, or whatever is the target's response to the probe
 
-The output for this host would then say "Hops 1-7 as the same as for ...".
+The output for this host would then say "Hops 1-7 are the same as for ...".
 
 The detection of shared traces rests on the assumption that all paths going
 through a router at a certain TTL will be identical up to and including the
@@ -137,10 +140,11 @@ individually.
 #include "NmapOps.h"
 #include "Target.h"
 
+#include "struct_ip.h"
+
 #include <dnet.h>
 
 #include <algorithm>
-#include <bitset>
 #include <list>
 #include <map>
 #include <set>
@@ -233,7 +237,7 @@ public:
   Target *target;
   /* A bitmap of TTLs that have been sent, to avoid duplicates when we switch
      around the order counting up or down. */
-  std::bitset<MAX_TTL + 1> sent_ttls;
+  std::vector<bool> sent_ttls;
   u8 current_ttl;
   enum counting_state state;
   /* If nonzero, the known hop distance to the target. */
@@ -285,7 +289,7 @@ public:
   void resend(int rawsd, eth_t *ethsd, struct timeval *now = NULL);
   bool is_timedout(struct timeval *now = NULL) const;
   bool may_resend() const;
-  virtual unsigned char *build_packet(const struct in_addr *source,
+  virtual unsigned char *build_packet(const struct sockaddr_storage *source,
     u32 *len) const = 0;
 
   static Probe *make(HostState *host, struct probespec pspec, u8 ttl);
@@ -331,7 +335,7 @@ static Hop *hop_cache_lookup(u8 ttl, const struct sockaddr_storage *addr);
 static void hop_cache_insert(Hop *hop);
 static unsigned int hop_cache_size();
 
-HostState::HostState(Target *target) {
+HostState::HostState(Target *target) : sent_ttls(MAX_TTL + 1, false) {
   this->target = target;
   current_ttl = MIN(MAX(1, HostState::distance_guess(target)), MAX_TTL);
   state = HostState::COUNTING_DOWN;
@@ -388,7 +392,7 @@ bool HostState::send_next_probe(int rawsd, eth_t *ethsd) {
   unanswered_probes.push_back(probe);
   active_probes.push_back(probe);
   probe->send(rawsd, ethsd);
-  sent_ttls.set(current_ttl);
+  sent_ttls[current_ttl] = true;
 
   return true;
 }
@@ -397,14 +401,14 @@ bool HostState::send_next_probe(int rawsd, eth_t *ethsd) {
 void HostState::next_ttl() {
   assert(current_ttl > 0);
   if (state == HostState::COUNTING_DOWN) {
-    while (current_ttl > 1 && sent_ttls.test(current_ttl))
+    while (current_ttl > 1 && sent_ttls[current_ttl])
       current_ttl--;
     if (current_ttl == 1)
       state = HostState::COUNTING_UP;
   }
   /* Note no "else". */
   if (state == HostState::COUNTING_UP) {
-    while (current_ttl <= MAX_TTL && sent_ttls.test(current_ttl))
+    while (current_ttl <= MAX_TTL && sent_ttls[current_ttl])
       current_ttl++;
   }
 }
@@ -511,10 +515,19 @@ void HostState::link_to(Hop *hop) {
 }
 
 double HostState::completion_fraction() const {
+  std::vector<bool>::iterator it;
+  unsigned int i, n;
+
   if (this->is_finished())
     return 1.0;
-  else
-    return (double) sent_ttls.count() / MAX_TTL;
+
+  n = 0;
+  for (i = 0; i < sent_ttls.size(); i++) {
+    if (sent_ttls[i])
+      n++;
+  }
+
+  return (double) n / sent_ttls.size();
 }
 
 void HostState::child_parent_ttl(u8 ttl, Hop **child, Hop **parent) {
@@ -542,13 +555,9 @@ struct probespec HostState::get_probe(const Target *target) {
   struct probespec probe;
 
   probe = target->pingprobe;
-  if (probe.type == PS_NONE || probe.type == PS_ARP) {
-    /* No responsive probe known? The user probably skipped both ping and
-       port scan. Guess ICMP echo as the most likely to get a response. */
-    probe.type = PS_ICMP;
-    probe.proto = IPPROTO_ICMP;
-    probe.pd.icmp.type = ICMP_ECHO;
-    probe.pd.icmp.code = 0;
+  if (probe.type == PS_TCP || probe.type == PS_UDP || probe.type == PS_ICMP ||
+      probe.type == PS_SCTP || probe.type == PS_ICMPV6) {
+    /* Nothing needed. */
   } else if (probe.type == PS_PROTO) {
     /* If this is an IP protocol probe, fill in some fields for some common
        protocols. We cheat and store them in the TCP-, UDP-, SCTP- and
@@ -563,6 +572,22 @@ struct probespec HostState::get_probe(const Target *target) {
     } else if (probe.proto == IPPROTO_ICMP) {
       probe.pd.icmp.type = ICMP_ECHO;
     }
+  } else {
+    /* No responsive probe known? The user probably skipped both ping and
+       port scan. Guess ICMP echo as the most likely to get a response. */
+    if (target->af() == AF_INET) {
+      probe.type = PS_ICMP;
+      probe.proto = IPPROTO_ICMP;
+      probe.pd.icmp.type = ICMP_ECHO;
+      probe.pd.icmp.code = 0;
+    } else if (target->af() == AF_INET6) {
+      probe.type = PS_ICMPV6;
+      probe.proto = IPPROTO_ICMPV6;
+      probe.pd.icmp.type = ICMPV6_ECHO;
+      probe.pd.icmp.code = 0;
+    } else {
+      fatal("Unknown address family %d", target->af());
+    }
   }
 
   return probe;
@@ -598,19 +623,30 @@ void Probe::send(int rawsd, eth_t *ethsd, struct timeval *now) {
   }
 
   for (decoy = 0; decoy < o.numdecoys; decoy++) {
-    const struct in_addr *source;
+    struct sockaddr_storage source;
+    size_t source_len;
     unsigned char *packet;
     u32 packetlen;
 
     if (decoy == o.decoyturn) {
-      source = host->target->v4sourceip();
+      source_len = sizeof(source);
+      host->target->SourceSockAddr(&source, &source_len);
       sent_time = get_now(now);
     } else {
-      source = &o.decoys[decoy];
+      if (o.af() == AF_INET) {
+        struct sockaddr_in *sin;
+
+        sin = (struct sockaddr_in *) &source;
+        sin->sin_family = AF_INET;
+        sin->sin_addr = o.decoys[decoy];
+      } else {
+        /* Decoys are IPv4-only. */
+        continue;
+      }
     }
 
-    packet = this->build_packet(source, &packetlen);
-    send_ip_packet(rawsd, ethp, packet, packetlen);
+    packet = this->build_packet(&source, &packetlen);
+    send_ip_packet(rawsd, ethp, host->target->TargetSockAddr(), packet, packetlen);
     free(packet);
   }
 }
@@ -642,8 +678,11 @@ public:
   : Probe(host, pspec, ttl) {
   }
 
-  unsigned char *build_packet(const struct in_addr *source, u32 *len) const {
-    return build_icmp_raw(source, host->target->v4hostip(), ttl,
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
+    const struct sockaddr_in *sin;
+    assert(source->ss_family == AF_INET);
+    sin = (struct sockaddr_in *) source;
+    return build_icmp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
       0x0000, 0x00, false, NULL, 0, token, global_id,
       pspec.pd.icmp.type, pspec.pd.icmp.code,
       o.extra_payload, o.extra_payload_length, len);
@@ -656,7 +695,7 @@ public:
   TCPProbe(HostState *host, struct probespec pspec, u8 ttl)
   : Probe(host, pspec, ttl) {
   }
-  unsigned char *build_packet(const struct in_addr *source, u32 *len) const {
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
     const char *tcpopts;
     int tcpoptslen;
     u32 ack;
@@ -673,11 +712,26 @@ public:
     }
 
     /* For TCP we encode the token in the source port. */
-    return build_tcp_raw(source, host->target->v4hostip(), ttl,
-      get_random_u16(), get_random_u8(), false, NULL, 0,
-      token ^ global_id, pspec.pd.tcp.dport, get_random_u32(), ack, 0x00,
-      pspec.pd.tcp.flags, get_random_u16(), 0, (const u8 *) tcpopts, tcpoptslen,
-      o.extra_payload, o.extra_payload_length, len);
+    if (source->ss_family == AF_INET) {
+      const struct sockaddr_in *sin = (struct sockaddr_in *) source;
+      return build_tcp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
+        get_random_u16(), get_random_u8(), false, NULL, 0,
+        token ^ global_id, pspec.pd.tcp.dport, get_random_u32(), ack, 0x00,
+        pspec.pd.tcp.flags, get_random_u16(), 0, (const u8 *) tcpopts, tcpoptslen,
+        o.extra_payload, o.extra_payload_length, len);
+    } else if (source->ss_family == AF_INET6) {
+      const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) source;
+      return build_tcp_raw_ipv6(&sin6->sin6_addr, host->target->v6hostip(),
+        0, 0, ttl,
+        token ^ global_id, pspec.pd.tcp.dport, get_random_u32(), ack, 0x00,
+        pspec.pd.tcp.flags, get_random_u16(), 0, (const u8 *) tcpopts, tcpoptslen,
+        o.extra_payload, o.extra_payload_length, len);
+    } else {
+      fatal("Unknown address family %u in %s.", source->ss_family, __func__);
+    }
+
+    /* This should not be reached. Just in case. */
+    assert(0);
   }
 };
 
@@ -687,14 +741,18 @@ public:
   UDPProbe(HostState *host, struct probespec pspec, u8 ttl)
   : Probe(host, pspec, ttl) {
   }
-  unsigned char *build_packet(const struct in_addr *source, u32 *len) const {
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
     const char *payload;
     size_t payload_length;
+    const struct sockaddr_in *sin;
+
+    assert(source->ss_family == AF_INET);
+    sin = (struct sockaddr_in *) source;
 
     payload = get_udp_payload(pspec.pd.udp.dport, &payload_length);
 
     /* For UDP we encode the token in the source port. */
-    return build_udp_raw(source, host->target->v4hostip(), ttl,
+    return build_udp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
       get_random_u16(), get_random_u8(), false, NULL, 0,
       token ^ global_id, pspec.pd.udp.dport,
       payload, payload_length, len);
@@ -707,12 +765,16 @@ public:
   SCTPProbe(HostState *host, struct probespec pspec, u8 ttl)
   : Probe(host, pspec, ttl) {
   }
-  unsigned char *build_packet(const struct in_addr *source, u32 *len) const {
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
     struct sctp_chunkhdr_init chunk;
+    const struct sockaddr_in *sin;
+
+    assert(source->ss_family == AF_INET);
+    sin = (struct sockaddr_in *) source;
 
     sctp_pack_chunkhdr_init(&chunk, SCTP_INIT, 0, sizeof(chunk),
       get_random_u32() /*itag*/, 32768, 10, 2048, get_random_u32() /*itsn*/);
-    return build_sctp_raw(source, host->target->v4hostip(), ttl,
+    return build_sctp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
       get_random_u16(), get_random_u8(), false, NULL, 0,
       token ^ global_id, pspec.pd.sctp.dport, 0UL,
       (char *) &chunk, sizeof(chunk),
@@ -726,14 +788,34 @@ public:
   IPProtoProbe(HostState *host, struct probespec pspec, u8 ttl)
   : Probe(host, pspec, ttl) {
   }
-  unsigned char *build_packet(const struct in_addr *source, u32 *len) const {
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
+    const struct sockaddr_in *sin;
+    assert(source->ss_family == AF_INET);
+    sin = (struct sockaddr_in *) source;
     /* For IP proto scan the token is put in the IP ID. */
-    return build_ip_raw(source, host->target->v4hostip(), pspec.proto, ttl,
+    return build_ip_raw(&sin->sin_addr, host->target->v4hostip(), pspec.proto, ttl,
       token ^ global_id, get_random_u8(), false, NULL, 0,
       o.extra_payload, o.extra_payload_length, len);
   }
 };
 
+class ICMPv6Probe : public Probe
+{
+public:
+  ICMPv6Probe(HostState *host, struct probespec pspec, u8 ttl)
+  : Probe(host, pspec, ttl) {
+  }
+
+  unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
+    const struct sockaddr_in6 *sin6;
+    assert(source->ss_family == AF_INET6);
+    sin6 = (struct sockaddr_in6 *) source;
+    return build_icmpv6_raw(&sin6->sin6_addr, host->target->v6hostip(), 0x00, 0x0000,
+      ttl, token, global_id, pspec.pd.icmp.type, pspec.pd.icmp.code,
+      o.extra_payload, o.extra_payload_length, len);
+  }
+};
+
 Probe *Probe::make(HostState *host, struct probespec pspec, u8 ttl)
 {
   if (pspec.type == PS_ICMP || (pspec.type == PS_PROTO && pspec.proto == IPPROTO_ICMP))
@@ -746,6 +828,8 @@ Probe *Probe::make(HostState *host, struct probespec pspec, u8 ttl)
     return new SCTPProbe(host, pspec, ttl);
   else if (pspec.type == PS_PROTO)
     return new IPProtoProbe(host, pspec, ttl);
+  else if (pspec.type == PS_ICMPV6)
+    return new ICMPv6Probe(host, pspec, ttl);
   else
     fatal("Unknown probespec type in traceroute");
 
@@ -768,7 +852,7 @@ TracerouteState::TracerouteState(std::vector<Target *> &targets) {
     rawsd = -1;
   } else {
 #ifdef WIN32
-    win32_warn_raw_sockets(targets[0]->deviceName());
+    win32_fatal_raw_sockets(targets[0]->deviceName());
 #endif
     rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
     if (rawsd == -1)
@@ -781,14 +865,16 @@ TracerouteState::TracerouteState(std::vector<Target *> &targets) {
   }
 
   /* Assume that all the targets share the same device. */
-  pd = my_pcap_open_live(targets[0]->deviceName(), 128, o.spoofsource, 2);
+  if((pd=my_pcap_open_live(targets[0]->deviceName(), 128, o.spoofsource, 2))==NULL)
+    fatal("%s", PCAP_OPEN_ERRMSG);
   sslen = sizeof(srcaddr);
   targets[0]->SourceSockAddr(&srcaddr, &sslen);
-  n = Snprintf(pcap_filter, sizeof(pcap_filter), "dst host %s",
+  n = Snprintf(pcap_filter, sizeof(pcap_filter), "(ip or ip6) and dst host %s",
     ss_to_string(&srcaddr));
   assert(n < (int) sizeof(pcap_filter));
   set_pcap_filter(targets[0]->deviceFullName(), pd, pcap_filter);
-
+ if (o.debugging)
+   log_write(LOG_STDOUT, "Packet capture filter (device %s): %s\n", targets[0]->deviceFullName(), pcap_filter);
   for (it = targets.begin(); it != targets.end(); it++) {
     HostState *state = new HostState(*it);
     hosts.push_back(state);
@@ -983,11 +1069,11 @@ void TracerouteState::set_host_hop(HostState *host, u8 ttl,
       while (hop->parent != NULL) {
         hop = hop->parent;
         /* No need to re-probe any merged hops. */
-        host->sent_ttls.set(hop->ttl);
+        host->sent_ttls[hop->ttl] = true;
       }
       sslen = sizeof(addr);
       host->target->TargetSockAddr(&addr, &sslen);
-      if (sockaddr_storage_cmp(&hop->tag, &addr) == 0) {
+      if (sockaddr_storage_equal(&hop->tag, &addr)) {
         if (o.debugging > 1) {
           log_write(LOG_STDOUT, "%s continuing trace from TTL %d\n",
             host->target->targetipstr(), host->current_ttl);
@@ -1014,113 +1100,81 @@ struct Reply {
   u16 token;
 };
 
-static const void *ip_get_data(const struct ip *ip, unsigned int *len)
-{
-  unsigned int header_len;
+static bool parse_encapsulated_reply(const void *ip, unsigned len, Reply *reply) {
+  struct abstract_ip_hdr hdr;
+  const void *data;
 
-  header_len = ip->ip_hl * 4;
-  if (header_len > *len)
-    return NULL;
-  *len -= header_len;
+  data = ip_get_data(ip, &len, &hdr);
+  if (data == NULL)
+    return false;
 
-  return (char *) ip + header_len;
-}
-
-static const void *icmp_get_data(const struct icmp *icmp, unsigned int *len)
-{
-  unsigned int header_len;
-
-  if (icmp->icmp_type == ICMP_TIMEXCEED || icmp->icmp_type == ICMP_UNREACH)
-    header_len = 8;
-  else
-    fatal("%s passed ICMP packet with unhandled type %d", __func__, icmp->icmp_type);
-  if (header_len > *len)
-    return NULL;
-  *len -= header_len;
-
-  return (char *) icmp + header_len;
-}
-
-static bool parse_encapsulated_reply(const struct ip *ip, unsigned int len,
-  Reply *reply) {
-  sockaddr_in *addr_in;
-
-  if (ip->ip_p == IPPROTO_ICMP) {
-    const struct icmp *icmp = (struct icmp *) ip_get_data(ip, &len);
-    if (icmp == NULL || len < 8 || ntohs(icmp->icmp_id) != global_id)
+  if (hdr.version == 4 && hdr.proto == IPPROTO_ICMP) {
+    const struct icmp *icmp = (const struct icmp *) data;
+    if (len < 8 || ntohs(icmp->icmp_id) != global_id)
       return false;
     reply->token = ntohs(icmp->icmp_seq);
-  } else if (ip->ip_p == IPPROTO_TCP) {
-    const struct tcp_hdr *tcp = (struct tcp_hdr *) ip_get_data(ip, &len);
-    if (tcp == NULL || len < 2)
+  } else if (hdr.version == 6 && hdr.proto == IPPROTO_ICMPV6) {
+    const struct icmpv6_msg_echo *echo = (struct icmpv6_msg_echo *) ((char *) data + sizeof(struct icmpv6_hdr));
+    if (len < 8 || ntohs(echo->icmpv6_id) != global_id)
+      return false;
+    reply->token = ntohs(echo->icmpv6_seq);
+  } else if (hdr.proto == IPPROTO_TCP) {
+    const struct tcp_hdr *tcp = (const struct tcp_hdr *) data;
+    if (len < 2)
       return false;
     reply->token = ntohs(tcp->th_sport) ^ global_id;
-  } else if (ip->ip_p == IPPROTO_UDP) {
-    const struct udp_hdr *udp = (struct udp_hdr *) ip_get_data(ip, &len);
-    if (udp == NULL || len < 2)
+  } else if (hdr.proto == IPPROTO_UDP) {
+    const struct udp_hdr *udp = (const struct udp_hdr *) data;
+    if (len < 2)
       return false;
     reply->token = ntohs(udp->uh_sport) ^ global_id;
-  } else if (ip->ip_p == IPPROTO_SCTP) {
-    const struct sctp_hdr *sctp = (struct sctp_hdr *) ip_get_data(ip, &len);
-    if (sctp == NULL || len < 2)
+  } else if (hdr.proto == IPPROTO_SCTP) {
+    const struct sctp_hdr *sctp = (const struct sctp_hdr *) data;
+    if (len < 2)
       return false;
     reply->token = ntohs(sctp->sh_sport) ^ global_id;
   } else {
     if (len < 6)
       return false;
     /* Check IP ID for proto scan. */
-    reply->token = ntohs(ip->ip_id) ^ global_id;
+    reply->token = hdr.ipid ^ global_id;
   }
 
-  addr_in = (struct sockaddr_in *) &reply->target_addr;
-  addr_in->sin_family = AF_INET;
-  addr_in->sin_addr = ip->ip_dst;
+  reply->target_addr = hdr.dst;
 
   return true;
 }
 
-static bool read_reply(Reply *reply, pcap_t *pd, long timeout) {
-  const struct ip *ip;
-  unsigned int iplen;
-  struct link_header linkhdr;
-  sockaddr_in *addr_in;
+static bool decode_reply(const void *ip, unsigned int len, Reply *reply) {
+  struct abstract_ip_hdr hdr;
+  const void *data;
 
-  ip = (struct ip *) readip_pcap(pd, &iplen, timeout, &reply->rcvdtime,
-    &linkhdr, true);
-  if (ip == NULL)
-    return false;
-  if (ip->ip_v != 4)
+  data = ip_get_data(ip, &len, &hdr);
+  if (data == NULL)
     return false;
 
-  addr_in = (struct sockaddr_in *) &reply->from_addr;
-  addr_in->sin_family = AF_INET;
-  addr_in->sin_addr = ip->ip_src;
+  reply->from_addr = hdr.src;
 
-  reply->ttl = ip->ip_ttl;
-
-  if (ip->ip_p == IPPROTO_ICMP) {
+  if (hdr.version == 4 && hdr.proto == IPPROTO_ICMP) {
     /* ICMP responses comprise all the TTL exceeded messages we expect from all
        probe types, as well as actual replies from ICMP probes. */
-    const struct icmp *icmp;
-    unsigned int icmplen;
-
-    icmplen = iplen;
-    icmp = (struct icmp *) ip_get_data(ip, &icmplen);
-    if (icmp == NULL || icmplen < 8)
+    const struct icmp_hdr *icmp = (const struct icmp_hdr *) data;
+    if (len < 8)
       return false;
     if ((icmp->icmp_type == ICMP_TIMEXCEED
          && icmp->icmp_code == ICMP_TIMEXCEED_INTRANS)
         || icmp->icmp_type == ICMP_UNREACH) {
-      /* Get the encapsulated ICMP packet. */
-      iplen = icmplen;
-      ip = (struct ip *) icmp_get_data(icmp, &iplen);
-      if (ip == NULL || ip->ip_v != 4 || iplen < 20)
-        return false;
-      if (!parse_encapsulated_reply(ip, iplen, reply))
+      /* Get the encapsulated IP packet. */
+      const void *encaps = icmp_get_data(icmp, &len);
+      if (encaps == NULL)
         return false;
+      return parse_encapsulated_reply(encaps, len, reply);
     } else if (icmp->icmp_type == ICMP_ECHOREPLY
                || icmp->icmp_type == ICMP_MASKREPLY
                || icmp->icmp_type == ICMP_TSTAMPREPLY) {
+      /* Need this alternate form of header for icmp_id and icmp_seq. */
+      const struct icmp *icmp = (const struct icmp *) data;
+
       if (ntohs(icmp->icmp_id) != global_id)
         return false;
       reply->token = ntohs(icmp->icmp_seq);
@@ -1129,33 +1183,51 @@ static bool read_reply(Reply *reply, pcap_t *pd, long timeout) {
     } else {
       return false;
     }
-  } else if (ip->ip_p == IPPROTO_TCP) {
-    const struct tcp_hdr *tcp;
-    unsigned int tcplen;
+  } else if (hdr.version == 6 && hdr.proto == IP_PROTO_ICMPV6) {
+    /* ICMPv6 responses comprise all the TTL exceeded messages we expect from
+       all probe types, as well as actual replies from ICMP probes. */
+    const struct icmpv6_hdr *icmpv6 = (const struct icmpv6_hdr *) data;
+    if (len < 2)
+      return false;
+    /* TIMEXCEED, UNREACH */
+    if ((icmpv6->icmpv6_type == ICMPV6_TIMEXCEED
+         && icmpv6->icmpv6_code == ICMPV6_TIMEXCEED_INTRANS)
+        || icmpv6->icmpv6_type == ICMPV6_UNREACH) {
+      /* Get the encapsulated IP packet. */
+      const void *encaps = icmpv6_get_data(icmpv6, &len);
+      if (encaps == NULL)
+        return false;
+      return parse_encapsulated_reply(encaps, len, reply);
+    } else if (icmpv6->icmpv6_type == ICMPV6_ECHOREPLY) {
+      /* MASKREPLY, TSTAMPREPLY */
+      const struct icmpv6_msg_echo *echo;
 
-    tcplen = iplen;
-    tcp = (struct tcp_hdr *) ip_get_data(ip, &tcplen);
-    if (tcp == NULL || tcplen < 4)
+      if (len < sizeof(*icmpv6) + 4)
+	return false;
+      echo = (struct icmpv6_msg_echo *) ((char *) icmpv6 + sizeof(*icmpv6));
+      if (ntohs(echo->icmpv6_id) != global_id)
+        return false;
+      reply->token = ntohs(echo->icmpv6_seq);
+      /* Reply came directly from the target. */
+      reply->target_addr = reply->from_addr;
+    } else {
+      return false;
+    }
+  } else if (hdr.proto == IPPROTO_TCP) {
+    const struct tcp_hdr *tcp = (const struct tcp_hdr *) data;
+    if (len < 4)
       return false;
     reply->token = ntohs(tcp->th_dport) ^ global_id;
     reply->target_addr = reply->from_addr;
-  } else if (ip->ip_p == IPPROTO_UDP) {
-    const struct udp_hdr *udp;
-    unsigned int udplen;
-
-    udplen = iplen;
-    udp = (struct udp_hdr *) ip_get_data(ip, &udplen);
-    if (udp == NULL || udplen < 4)
+  } else if (hdr.proto == IPPROTO_UDP) {
+    const struct udp_hdr *udp = (const struct udp_hdr *) data;
+    if (len < 4)
       return false;
     reply->token = ntohs(udp->uh_dport) ^ global_id;
     reply->target_addr = reply->from_addr;
-  } else if (ip->ip_p == IPPROTO_SCTP) {
-    const struct sctp_hdr *sctp;
-    unsigned int sctplen;
-
-    sctplen = iplen;
-    sctp = (struct sctp_hdr *) ip_get_data(ip, &sctplen);
-    if (sctp == NULL || sctplen < 4)
+  } else if (hdr.proto == IPPROTO_SCTP) {
+    const struct sctp_hdr *sctp = (const struct sctp_hdr *) data;
+    if (len < 4)
       return false;
     reply->token = ntohs(sctp->sh_dport) ^ global_id;
     reply->target_addr = reply->from_addr;
@@ -1166,6 +1238,20 @@ static bool read_reply(Reply *reply, pcap_t *pd, long timeout) {
   return true;
 }
 
+static bool read_reply(Reply *reply, pcap_t *pd, long timeout) {
+  const struct ip *ip;
+  unsigned int iplen;
+  struct link_header linkhdr;
+
+  ip = (struct ip *) readip_pcap(pd, &iplen, timeout, &reply->rcvdtime, &linkhdr, true);
+  if (ip == NULL)
+    return false;
+  if (ip->ip_v == 4 || ip->ip_v == 6)
+    return decode_reply(ip, iplen, reply);
+  else
+    return false;
+}
+
 void TracerouteState::read_replies(long timeout) {
   struct sockaddr_storage ss;
   struct timeval now;
@@ -1194,7 +1280,7 @@ void TracerouteState::read_replies(long timeout) {
 
     sslen = sizeof(ss);
     host->target->TargetSockAddr(&ss, &sslen);
-    if (sockaddr_storage_cmp(&ss, &reply.from_addr) == 0) {
+    if (sockaddr_storage_equal(&ss, &reply.from_addr)) {
       adjust_timeouts2(&probe->sent_time, &reply.rcvdtime, &host->target->to);
       if (host->reached_target == 0 || probe->ttl < host->reached_target)
         host->reached_target = probe->ttl;
@@ -1255,12 +1341,19 @@ void TracerouteState::remove_finished_hosts() {
   }
 }
 
+/* Dummy class to use sockaddr_storage as a map key. */
+struct lt_sockaddr_storage {
+  bool operator()(const struct sockaddr_storage& a, const struct sockaddr_storage& b) const {
+    return sockaddr_storage_cmp(&a, &b) < 0;
+  }
+};
+
 /* Find the reverse-DNS names of the hops. */
 void TracerouteState::resolve_hops() {
-  std::set<uint32_t> addrs;
-  std::set<uint32_t>::iterator addr_iter;
+  std::set<sockaddr_storage, lt_sockaddr_storage> addrs;
+  std::set<sockaddr_storage, lt_sockaddr_storage>::iterator addr_iter;
   std::vector<HostState *>::iterator host_iter;
-  std::map<uint32_t, const char *> name_map;
+  std::map<sockaddr_storage, const char *, lt_sockaddr_storage> name_map;
   Target **targets;
   Hop *hop;
   int i, n;
@@ -1270,9 +1363,8 @@ void TracerouteState::resolve_hops() {
      inefficient. */
   for (host_iter = hosts.begin(); host_iter != hosts.end(); host_iter++) {
     for (hop = (*host_iter)->hops; hop != NULL; hop = hop->parent) {
-      struct sockaddr_in *sin = (struct sockaddr_in *) &hop->addr;
-      if (hop->addr.ss_family == AF_INET)
-        addrs.insert(sin->sin_addr.s_addr);
+      if (hop->addr.ss_family != AF_UNSPEC)
+        addrs.insert(hop->addr);
     }
   }
   n = addrs.size();
@@ -1282,11 +1374,8 @@ void TracerouteState::resolve_hops() {
   i = 0;
   addr_iter = addrs.begin();
   while (i < n) {
-    struct sockaddr_in sin;
-    sin.sin_family = AF_INET;
-    sin.sin_addr.s_addr = *addr_iter;
     targets[i] = new Target();
-    targets[i]->setTargetSockAddr((struct sockaddr_storage *) &sin, sizeof(sin));
+    targets[i]->setTargetSockAddr(&*addr_iter, sizeof(*addr_iter));
     targets[i]->flags = HOST_UP;
     i++;
     addr_iter++;
@@ -1294,17 +1383,20 @@ void TracerouteState::resolve_hops() {
   nmap_mass_rdns(targets, n);
   /* Third, make a map from addresses to names for easy lookup. */
   for (i = 0; i < n; i++) {
+    struct sockaddr_storage ss;
+    size_t ss_len;
     const char *hostname = targets[i]->HostName();
     if (*hostname == '\0')
       hostname = NULL;
-    name_map[targets[i]->v4hostip()->s_addr] = hostname;
+    ss_len = sizeof(ss);
+    targets[i]->TargetSockAddr(&ss, &ss_len);
+    name_map[ss] = hostname;
   }
   /* Finally, copy the names into the hops. */
   for (host_iter = hosts.begin(); host_iter != hosts.end(); host_iter++) {
     for (hop = (*host_iter)->hops; hop != NULL; hop = hop->parent) {
-      struct sockaddr_in *sin = (struct sockaddr_in *) &hop->addr;
-      if (hop->addr.ss_family == AF_INET) {
-        const char *hostname = name_map[sin->sin_addr.s_addr];
+      if (hop->addr.ss_family != AF_UNSPEC) {
+        const char *hostname = name_map[hop->addr];
         if (hostname != NULL)
           hop->hostname = hostname;
       }
@@ -1361,7 +1453,7 @@ Probe *TracerouteState::lookup_probe(
 
     sslen = sizeof(ss);
     (*host_iter)->target->TargetSockAddr(&ss, &sslen);
-    if (sockaddr_storage_cmp(&ss, target_addr) != 0)
+    if (!sockaddr_storage_equal(&ss, target_addr))
       continue;
     for (probe_iter = (*host_iter)->unanswered_probes.begin();
          probe_iter != (*host_iter)->unanswered_probes.end();
@@ -1384,68 +1476,6 @@ double TracerouteState::completion_fraction() const {
   return sum / hosts.size();
 }
 
-static int traceroute_direct(std::vector<Target *> targets);
-
-int traceroute(std::vector<Target *> &Targets) {
-  std::vector<Target *>::iterator target_iter;
-
-  if (Targets.empty() || Targets[0]->ifType() == devt_loopback)
-    return 1;
-
-  if (Targets[0]->directlyConnected())
-    /* Special case for directly connected targets. */
-    return traceroute_direct(Targets);
-
-  TracerouteState global_state(Targets);
-
-  global_id = get_random_u16();
-
-  ScanProgressMeter SPM("Traceroute");
-
-  o.current_scantype = TRACEROUTE;
-
-  while (!global_state.active_hosts.empty()) {
-    struct timeval now;
-    long int timeout;
-
-    global_state.send_new_probes();
-    now = get_now();
-    timeout = TIMEVAL_SUBTRACT(global_state.next_send_time, now);
-    global_state.read_replies(timeout);
-    global_state.cull_timeouts();
-    global_state.remove_finished_hosts();
-
-    if (keyWasPressed())
-      SPM.printStats(global_state.completion_fraction(), NULL);
-  }
-
-  SPM.endTask(NULL, NULL);
-
-  if (!o.noresolve)
-    global_state.resolve_hops();
-  /* This puts the hops into the Targets known by the global_state. */
-  global_state.transfer_hops();
-
-  /* Update initial_ttl to be the highest distance seen in this host group, as
-     an estimate for the next. */
-  initial_ttl = 0;
-  for (target_iter = Targets.begin();
-       target_iter != Targets.end();
-       target_iter++) {
-    initial_ttl = MAX(initial_ttl, (*target_iter)->traceroute_hops.size());
-  }
-
-  if (hop_cache_size() > MAX_HOP_CACHE_SIZE) {
-    if (o.debugging) {
-      log_write(LOG_STDOUT, "Clearing hop cache that has grown to %d\n",
-        hop_cache_size());
-    }
-    traceroute_hop_cache_clear();
-  }
-
-  return 1;
-}
-
 /* This is a special case of traceroute when all the targets are directly
    connected. Because the distance to each target is known to be 1, we send no
    probes at all, only fill in a TracerouteHop structure. */
@@ -1472,6 +1502,84 @@ static int traceroute_direct(std::vector<Target *> targets) {
   return 1;
 }
 
+static int traceroute_remote(std::vector<Target *> targets) {
+  std::vector<Target *>::iterator target_iter;
+
+  if (targets.empty())
+    return 1;
+
+  TracerouteState global_state(targets);
+
+  global_id = get_random_u16();
+
+  ScanProgressMeter SPM("Traceroute");
+
+  o.current_scantype = TRACEROUTE;
+
+  while (!global_state.active_hosts.empty()) {
+    struct timeval now;
+    long int timeout;
+
+    global_state.send_new_probes();
+    now = get_now();
+    timeout = TIMEVAL_SUBTRACT(global_state.next_send_time, now);
+    global_state.read_replies(timeout);
+    global_state.cull_timeouts();
+    global_state.remove_finished_hosts();
+
+    if (keyWasPressed())
+      SPM.printStats(global_state.completion_fraction(), NULL);
+  }
+
+  SPM.endTask(NULL, NULL);
+
+  if (!o.noresolve)
+    global_state.resolve_hops();
+  /* This puts the hops into the targets known by the global_state. */
+  global_state.transfer_hops();
+
+  /* Update initial_ttl to be the highest distance seen in this host group, as
+     an estimate for the next. */
+  initial_ttl = 0;
+  for (target_iter = targets.begin();
+       target_iter != targets.end();
+       target_iter++) {
+    initial_ttl = MAX(initial_ttl, (*target_iter)->traceroute_hops.size());
+  }
+
+  if (hop_cache_size() > MAX_HOP_CACHE_SIZE) {
+    if (o.debugging) {
+      log_write(LOG_STDOUT, "Clearing hop cache that has grown to %d\n",
+        hop_cache_size());
+    }
+    traceroute_hop_cache_clear();
+  }
+
+  return 1;
+}
+
+int traceroute(std::vector<Target *> &Targets) {
+  std::vector<Target *> direct, remote;
+  std::vector<Target *>::iterator target_iter;
+
+  /* Separate directly connected targets from remote targets. */
+  for (target_iter = Targets.begin();
+       target_iter != Targets.end();
+       target_iter++) {
+    if ((*target_iter)->ifType() == devt_loopback)
+      ; /* Ignore */
+    else if ((*target_iter)->directlyConnected())
+      direct.push_back(*target_iter);
+    else
+      remote.push_back(*target_iter);
+  }
+
+  traceroute_direct(direct);
+  traceroute_remote(remote);
+
+  return 1;
+}
+
 static struct timeval get_now(struct timeval *now) {
   struct timeval tv;
 
