@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
- 
+
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
 # * also a registered trademark of Insecure.Com LLC.  This program is free  *
 # * software; you may redistribute and/or modify it under the terms of the  *
 # * GNU General Public License as published by the Free Software            *
@@ -13,11 +13,12 @@
 # * technology into proprietary software, we sell alternative licenses      *
 # * (contact sales@insecure.com).  Dozens of software vendors already       *
 # * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, and version detection.                                       *
+# * detection, version detection, and the Nmap Scripting Engine.            *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derived works", yet *
 # * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we consider an application to constitute a           *
+# * misunderstandings, we interpret that term as broadly as copyright law   *
+# * allows.  For example, we consider an application to constitute a        *
 # * "derivative work" for the purpose of this license if it does any of the *
 # * following:                                                              *
 # * o Integrates source code from Nmap                                      *
@@ -25,31 +26,32 @@
 # *   nmap-os-db or nmap-service-probes.                                    *
 # * o Executes Nmap and parses the results (as opposed to typical shell or  *
 # *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                * 
+# *   not derivative works.)                                                *
 # * o Integrates/includes/aggregates Nmap into a proprietary executable     *
 # *   installer, such as those produced by InstallShield.                   *
 # * o Links to a library or executes a program that does any of the above   *
 # *                                                                         *
 # * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
-# * interpretation of derived works with some common examples.  Our         *
-# * interpretation applies only to Nmap--we don't speak for other people's  *
-# * GPL works.                                                              *
+# * works of Nmap, as well as other software we distribute under this       *
+# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+# * but is meant to clarify our interpretation of derived works with some   *
+# * common examples.  Our interpretation applies only to Nmap--we don't     *
+# * speak for other people's GPL works.                                     *
 # *                                                                         *
 # * If you have any questions about the GPL licensing restrictions on using *
 # * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 # * we also offer alternative license to integrate Nmap into proprietary    *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates as well as helping to     *
-# * fund the continued development of Nmap technology.  Please email        *
-# * sales@insecure.com for further information.                             *
+# * as providing for priority support and updates.  They also fund the      *
+# * continued development of Nmap.  Please email sales@insecure.com for     *
+# * further information.                                                    *
 # *                                                                         *
 # * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included COPYING.OpenSSL file, and distribute linked      *
-# * combinations including the two. You must obey the GNU GPL in all        *
+# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+# * linked combinations including the two. You must obey the GNU GPL in all *
 # * respects for all of the code used other than OpenSSL.  If you modify    *
 # * this file, you may extend this exception to your version of the file,   *
 # * but you are not obligated to do so.                                     *
@@ -67,15 +69,16 @@
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to nmap-dev@insecure.org for possible incorporation into the main       *
 # * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, it is assumed that you are      *
-# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
-# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because the *
-# * inability to relicense code has caused devastating problems for other   *
-# * Free Software projects (such as KDE and NASM).  We also occasionally    *
-# * relicense the code to third parties as discussed above.  If you wish to *
-# * specify special license conditions of your contributions, just say so   *
-# * when you send them.                                                     *
+# * Insecure.Org development mailing lists, or checking them into the Nmap  *
+# * source code repository, it is understood (unless you specify otherwise) *
+# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+# * code.  Nmap will always be available Open Source, but this is important *
+# * because the inability to relicense code has caused devastating problems *
+# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * occasionally relicense the code to third parties as discussed above.    *
+# * If you wish to specify special license conditions of your               *
+# * contributions, just say so when you send them.                          *
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -126,7 +129,7 @@ def mo_find(result, dirname, fnames):
         p = os.path.join(dirname, f)
         if os.path.isfile(p) and f.endswith(".mo"):
             files.append(p)
-        
+
     if files:
         result.append((dirname, files))
 
@@ -143,7 +146,7 @@ data_files = [ (pixmaps_dir, glob(os.path.join(pixmaps_dir, '*.gif')) +
                             [os.path.join(config_dir, 'scan_profile.usp')] +
                             [os.path.join(config_dir, APP_NAME + '_version')]),
 
-               (misc_dir, glob(os.path.join(misc_dir, '*.xml'))), 
+               (misc_dir, glob(os.path.join(misc_dir, '*.xml'))),
 
                (docs_dir, [os.path.join(docs_dir, 'help.html')])]
 
@@ -192,7 +195,7 @@ def path_strip_prefix(path, prefix):
 
 class my_install(install):
     def finalize_options(self):
-	# Ubuntu's python2.6-2.6.4-0ubuntu3 package changes sys.prefix in
+        # Ubuntu's python2.6-2.6.4-0ubuntu3 package changes sys.prefix in
         # install.finalize_options when sys.prefix is "/usr/local" (our
         # default). Because we need the unchanged value later, remember it here.
         self.saved_prefix = self.prefix
@@ -309,26 +312,24 @@ for dir in dirs:
         # Find where the modules are installed. distutils will put them in
         # self.install_lib, but that path can contain the root (DESTDIR), so we
         # must strip it off if necessary.
-        modules = self.install_lib
+        modules_dir = self.install_lib
         if self.root is not None:
-            modules = path_strip_prefix(modules, self.root)
+            modules_dir = path_strip_prefix(modules_dir, self.root)
 
-        ufile = open(app_file_name, "r")
-        ucontent = ufile.readlines()
-        ufile.close()
+        app_file = open(app_file_name, "r")
+        lines = app_file.readlines()
+        app_file.close()
 
-	# Insert our custom import after the first non-comment line.
-        re_sys = re.compile("^#")
-        uline = 0
-        for line in ucontent:
-            if not re_sys.match(line):
+        for i in range(len(lines)):
+            if re.match(r'^INSTALL_LIB =', lines[i]):
+                lines[i] = "INSTALL_LIB = %s\n" % repr(modules_dir)
                 break
-            uline += 1
-        ucontent.insert(uline, "\nimport sys\nsys.path.append(%s)\n" % repr(modules))
+        else:
+            raise ValueError("INSTALL_LIB replacement not found in %s" % app_file_name)
 
-        ufile = open(app_file_name, "w")
-        ufile.writelines(ucontent)
-        ufile.close()
+        app_file = open(app_file_name, "w")
+        app_file.writelines(lines)
+        app_file.close()
 
     def set_perms(self):
         re_bin = re.compile("(bin|\.sh)")
@@ -355,7 +356,9 @@ for dir in dirs:
                              "DOCS_DIR": os.path.join(self.saved_prefix, docs_dir),
                              "LOCALE_DIR": os.path.join(self.saved_prefix, locale_dir),
                              "MISC_DIR": os.path.join(self.saved_prefix, misc_dir),
-                             "PIXMAPS_DIR": os.path.join(self.saved_prefix, pixmaps_dir)}
+                             "PIXMAPS_DIR": os.path.join(self.saved_prefix, pixmaps_dir),
+                             # See $(nmapdatadir) in nmap/Makefile.in.
+                             "NMAPDATADIR": os.path.join(self.saved_prefix, "share", "nmap"),}
 
         # Find and read the Paths.py file.
         pcontent = ""
@@ -528,20 +531,12 @@ if 'py2exe' in sys.argv:
         'console': [{"script": "../ndiff/ndiff", "description": "Nmap scan comparison tool"}],
         'options': {"py2exe": {
             "compressed": 1,
-            "optimize":2,
-            "packages":"encodings",
-            "includes" : "\
-pango,\
-atk,\
-gobject,\
-pickle,\
-bz2,\
-encodings,\
-encodings.*,\
-cairo,\
-pangocairo,\
-atk\
-"}}
+            "optimize": 2,
+            "packages": ["encodings"],
+            "includes": ["pango", "atk", "gobject", "gio", "pickle", "bz2", "encodings", "encodings.*", "cairo", "pangocairo"],
+            "dll_excludes": ["USP10.dll", "NSI.dll", "MSIMG32.dll", "DNSAPI.dll"]
+            }
+        }
     }
 
     setup_args.update(WINDOWS_SETUP_ARGS)
